<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version
 * @author Daniel Perez	danielperez.arg@gmail.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
 
/**
 * A collection of system utilities
 */
class StringUtils {
	
	/**
	 * removeInvalidChars() - removes invalid chars from a given name
	 *
	 * @param		string	The name to check
	 * @return 	string	The name with the junk chars removed
	 */
	static function removeInvalidChars($name) {
		//return eregi_replace("[^-A-Z0-9_\.]",'',$name);
		// Read as: single back slash, comma, single quote, double quote, forward slash, blank
		// characters, at sign, asterisk, opening parenthesis, closing parenthesis, dollar sign,
		// ampersand, numeral, question marks, exclamation marks, backtick

		//$free = preg_replace('/[\\\\,\'"\\/[:blank:]@\\*()$&#�?\[\]�!`]/', '', $name);
		
		// For some reason preg_replace can do strange things with weird characters...  use str_replace instead
		$free = $name;
		$free = str_replace('\\', '', $free);
		$free = str_replace(',', '', $free);
		$free = str_replace('\'', '', $free);
		$free = str_replace('"', '', $free);
		$free = str_replace('/', '', $free);
		$free = str_replace(' ', '', $free);
		$free = str_replace('\t', '', $free);
		$free = str_replace('@', '', $free);
		$free = str_replace('*', '', $free);
		$free = str_replace('(', '', $free);
		$free = str_replace(')', '', $free);
		$free = str_replace('[', '', $free);
		$free = str_replace(']', '', $free);
		$free = str_replace('$', '', $free);
		$free = str_replace('&', '', $free);
		$free = str_replace('#', '', $free);
		$free = str_replace('¿', '', $free);
		$free = str_replace('?', '', $free);
		$free = str_replace('¡', '', $free);
		$free = str_replace('!', '', $free);
		$free = str_replace('`', '', $free);
		
		// Remove double dots
		$free = str_replace('..', '', $free);
		return $free;
	}
	
	/**
	 * Convert any string to a unix-system-like string (i.e. "This is a string" to "this-is-a-string")
	 * @param	string
	 * @return	string
	 */
	public static function generateUnixString($string) {
		$string = strtolower($string);
		$string = str_replace(' ', '-', $string);
		$string = self::removeInvalidChars($string);
		
		return $string;
	}

	/**
	 * prepare the received hash string from the browser
	 *
	 * @param string $hash
	 * @return sanitized string
	 */
	static function prepareHash($hash) {
           if (substr($hash, strlen($hash)-1, 1) == '>') {
                   $hash = substr($hash, 0, strlen($hash)-1);
           }
           if (substr($hash,0,1) == '_') {
                   $hash = substr($hash,1);
           }           
           return $hash;
	}
	
	/** 
	 * Converts the HTML special chars like &ntilde; into the corresponding UTF8 chars, like "ñ"
	 */
	static function unHtmlSpecialChars($str) {
		$translation_table_iso = get_html_translation_table(HTML_ENTITIES);
		$translation_table_utf8 = array();			// PHP returns the translation table in ISO-8859-1 instead of UTF8
		foreach ($translation_table_iso as $charkey => $char) {
			$charkey = utf8_encode($charkey);
			$translation_table_utf8[$charkey] = $char;
		}
		
		$tr_tbl = array_flip ($translation_table_utf8);
		$str = strtr ($str, $tr_tbl);
		// Following line is needed?
		$str = str_replace("&amp;quot;",'"',$str);
		return $str;
	}
	
	/**
	 * Convers an HTML string into its plain-text representation encoded in UTF-8. For example, turns
	 * "<i>se&ntilde;or<i>" into "señor". $str may not be in UTF-8.
	 */
	static function getPlainUTF8FromHtml($str) {
		$str = self::toValidUTF8($str);		
		$str = self::stripHtmlTags($str);
		$str = self::unHtmlSpecialChars($str);

		return $str;
	}
	
	static function removeLineBreaks($string) {
		$string = str_replace("\015", ' ', $string);	// CR
		$string = str_replace("\012", '', $string);		// LF
		return $string;
	}
	
	/**
	 * Given a list of e-mail addresses, returns an array with the individual addresses.
	 * It automatically detects the list's separator (comma, semicolon or space)
	 * @param	string $list
	 * @return	array
	 */
	static function getAddressesFromList($list) {
		$list = trim($list);
		if (empty($list)) return array();
		
		if (strpos($list, ',') !== false) {
			$addresses = explode(',', $list);
		} elseif (strpos($list, ';') !== false) {
			$addresses = explode(';', $list);
		} else {
			$addresses = explode(' ', $list);
		}
		
		// remove spaces
		$addresses = array_map('trim', $addresses);
		return $addresses;
	}
	
	/**
	 * Given a text, replaces occurences of http://some_link with <a href="http://some_link">http://some_link</a>
	 * @param	string
	 * @return	string
	 */
	static function addHtmlLinks($text) {
		preg_match_all('/(https?|ftp):\\/\\/[^[:blank:][:cntrl:]<>"\']*/i', $text, $matches);
		
		if (is_array($matches[0]) && count($matches[0]) > 0) {
			$added_links = array();			// this will avoid nesting links
			
			foreach ($matches[0] as $link) {
				if (preg_match('/\\.\\.\\.$/i', $link)) {
					// This ensures that link wasn't wrapped. For example, $text may end
					// with "see http://www.yahoo..."
					continue;
				}
				// remove last dot, if any (may be confused with part of the URL)
				$link = preg_replace('/\\.$/', '', $link);
				if (array_search($link, $added_links) === false) {		// link was not replaced
					$text = str_replace($link, '<a href="'.$link.'">'.$link.'</a>', $text);
					$added_links[] = $link;
				}
			}
		}

		return $text;
	}
	
	/**
	 * This method is used for extracting a string that is between two markers. Usually
	 * used when parsing replies in e-mails.
	 * @param	string $fulltext	Text where to search the string
	 * @param	string $beginmarker	Begin marker
	 * @param	string $endmarker	End marker
	 */
	public function getTextBetweenMarkers($fulltext, $beginmarker, $endmarker) {
		$firstpos = strpos($fulltext, $beginmarker);
		if ($firstpos === false) {
			// Marker not found on full text... assume no text
			return '';
		}
		// Move to the beginning of the text
		$firstpos += strlen($beginmarker);

		$lastpos = strpos($fulltext, $endmarker, $firstpos);
		if ($lastpos === false) {
			// Last marker not found. Assumme all the text below the marker is the string.
			$lastpos = strlen($fulltext)-1;
		}
		
		$text = substr($fulltext, $firstpos, $lastpos-$firstpos);
		$text = trim($text);
		
		return $text;
	}
	
	/**
	 * This method returns the error description of Uploaded files
	 * @param   string  $errorcode	Error code number
	 * @param   string  $filename	File name
	 * @return  string		Error description
	 */
	public function getFileUploadErrorString($errorcode, $filename) {
		switch ($errorcode) {
		    //The uploaded file exceeds the upload_max_filesize directive in php.ini
		    case 1:
			$out = array(Language::getSessText('FileUpload.MaxSize', array($filename)));
			break;
		    //The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form
		    case 2:
			$out = array(Language::getSessText('FileUpload.MaxSize', array($filename)));
			break;
		    //The uploaded file was only partially uploaded
		    case 3:
			$out = array(Language::getSessText('FileUpload.Partial', array($filename)));
			break;
		    case 4:
		    //the name of the file specified is not correct
			$out = array(Language::getSessText('FileUpload.InvalidFilename', array($filename)));
			break;
		    //Missing a temporary folder
		    case 6:
			$out = array(Language::getSessText('FileUpload.MissTempFolder', array($filename)));
			break;
		    //Failed to write file to disk
		    case 7:
			$out = array(Language::getSessText('FileUpload.WriteFailed', array($filename)));
			break;
		    default:
			break;
		}
		return $out;
	}
	
	public static function strReplaceOnce($search, $replace, $subject) {
	    $pos = strpos($subject, $search);
	    if($pos === false) {
		    return $subject;
	    } else {
			$start = substr($subject, 0, $pos);
			$end = substr($subject, $pos + strlen($search));
			return $start . $replace . $end;
	    }
	}
	
	/**
	 * Removes HTML tags from the text, and replaces <br> and <p> with newlines
	 */
	public static function stripHTMLTags($text) {
		$text = str_replace("<br />\n","\n",$text);
		$text = str_replace("<br />\r","\r",$text);
		$text = preg_replace('/<br[^>]*>/i', "\n", $text);
		$text = preg_replace('/<p[^>]*>/i', "\n", $text);
		$text = strip_tags($text);
		return $text;
	}
	
	/**
	 * Trims a string. Adds "..." at the end of the string if needed.
	 * @param	string $string
	 * @param	int $length
	 */
	public static function trimString($string, $length) {
		if (strlen($string) > $length) {
			return substr($string, 0, $length-3).'...';
		} else {
			return $string;
		}
	}
	
	/**
	 * Removes characters that may interfer with tooltipos
	 * @param	string $string
	 */
	public static function generateTooltip($string) {
		$string = nl2br($string);
		$string = str_replace("\n", "", $string);
		$string = str_replace("\r", "", $string);
		// Replace single quotes with \'
		$string = preg_replace('/([^\\\\])\'/', '$1\\\'', $string);
		$string = preg_replace('/^\'/', '\\\'', $string);
		// For some reason a text like "&lt;b&gt;foo&lt;/b&gt;" will render as bold on
		// the tooltip, so we htmlspecialchars() it again...
		$string = htmlspecialchars($string);
		return $string;
	}
	
	public static function extractDataFromLink($text)	{
		$pattern = "/href *= *['\"]?([^ >'\"]*).*?>(.*)</";
		preg_match($pattern, $text, $matches);
		$ret = Array('href' => $matches[1], 'text' => $matches[2]);
		return $ret;
	}
	
	/**
	* Returns "true" if param $str is an utf-8 encoded string, "false" otherwise
	*/
	public function isUTF8($str) {
		$c = 0;
		$b = 0;
		$bits = 0;
		$len=strlen($str);
		for ($i=0;$i < $len;$i++) {
    	$c = ord($str[$i]);
			if ($c > 128) {
				if ($c >= 254) {
					return false;
				} elseif ($c >= 252) {
					$bits=6;
				}	elseif ($c >= 248) {
					$bits=5;
				}	elseif ($c >= 240) {
					$bits=4;
				}	elseif ($c >= 224) {
					$bits=3;
				}	elseif ($c >= 192) {
					$bits=2;
				}	else {
					return false;
				}
				if (($i+$bits) > $len) {
					return false;
				}
				while ($bits > 1) {
					$i++;
					$b = ord($str[$i]);
					if ($b < 128 || $b > 191) {
						return false;
					}
					$bits--;
				}
			}
		}
		return true;
	}
	
	/**
	 * Extract relations embedded on a string. For example, "Commit against item [#1234] and FRS release [r6666]"
	 * Returns an array with items like "[r1234]" => array(section, ID)
	 */
	public function extractAssociationsFromString($string) {
		preg_match_all('/\\[#([rfdw]?)(\d+)\\]/', $string, $matches);
		
		$res = array();
		for ($i = 0; $i < count($matches[0]); $i++) {
			$relation_string = $matches[0][$i];
			$marker = $matches[1][$i];
			$id = $matches[2][$i];
			
			switch ($marker) {
			case 'r':
				$section = FrsRelease::SECTION;
				break;
			case 'f':
				$section = ForumMessage::SECTION;
				break;
			case 'd':
				$section = DocmanFile::SECTION;
				break;
			case 'w':
				$section = WikiPage::SECTION;
				break;
			default:
				$section = TrackerItem::SECTION;
				break;
			}
			
			$res[$relation_string] = array($section, $id);
		}
		return $res;
	}
	
	/**
	 * Converts *anything* to a valid UTF8 string. If it's already an UTF8 string, it returns the original string.
	 */
	public function toValidUTF8($string) {
		if ($string) {
			// No need to call mb_detect_encoding?
			if (!mb_check_encoding($string, 'UTF-8')) {
				$string = utf8_encode($string);
			}
		}
		
		/*if($strings){
			$current_encoding = mb_detect_encoding($strings,null,true);
			if($current_encoding != 'UTF-8' && !(mb_check_encoding($strings, 'UTF-8')) ){
				$strings = utf8_encode($strings);
			}
		}*/
		
		return $string;
	}

	/**
	 * Verrifies if a string is UTF8 compliant
	 */	
	public function isUTFf8Compliant($str) {
		if ( strlen($str) == 0 ) {
			return TRUE;
		}
			return (preg_match('/^.{1}/us',$str,$ar) == 1);
	}
	
}
?>
