<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */


class HTMLWidget {
	const OPTION_NONE = 100;
	const OPTION_ANY = '';

	/**
	 *	select - build a select box from the array of objects and selected item.
	 *	@param	string	The name of the widget.
	 *	@param	array	The array of objects, which must implement GFWidget, to be displayed.
	 *	@param	object	The optional object, which must implement GFWidget, that is selected (or its ID).
	 *	@param	string	The id of the default selected item.
	 *	@param	boolean	Whether to show a 'None' row.
	 *	@param	string	The text to show if you show a 'None' row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 */
	static function select($name,
		$objects,
		$selected_object = null,
		$default_selected = HTMLWidget::OPTION_NONE,
		$show_none = true, 
		$text_none = null,
		$show_any = false, 
		$text_any = null) {
	
		$language = Language::getSessLang();

		if ($show_none && is_null($text_none)) {
			$text_none = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}

		if (!$selected_object) {
			$selected = $default_selected;
		} elseif (is_object($selected_object)) {
			$selected = $selected_object->getID();
		} else {
			$selected = $selected_object;
		}

		$ret = '<select name="'.$name.'">';
		if ($show_any) {
			$ret .= '
				<option value="'.HTMLWidget::OPTION_ANY.'">'.$text_any.'</option>';
		}
		if ($show_none) {
			$ret .= '
				<option value="'.HTMLWidget::OPTION_NONE.'"'.(($selected == HTMLWidget::OPTION_NONE) ? ' selected="selected"' : '' ).'>'.$text_none.'</option>';
		}

		foreach ($objects as $object) {
			$id = $object->getID();
			$ret .= '
				<option value="'.$object->getID().'"'.(("$selected" == "$id") ? ' selected="selected" ' : '' ).'>'.$object->getName().'</option>';
			// Note the quotes above. It solves this:
			// http://bugs.php.net/bug.php?id=6766
		}
		$ret .= '</select>';

		return $ret;
	}

	/**
	 *	radio - build a radiobutton list from the array of objects and selected item.
	 *	@param	string	The name of the widget.
	 *	@param	array	The array of objects, which must implement GFWidget, to be displayed.
	 *	@param	object	The optional object, which must implement GFWidget, that is selected (or its ID).
	 *	@param	string	The id of the default selected item.
	 *	@param	boolean	Whether to show a 'None' row.
	 *	@param	string	The text to show if you show a 'None' row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 */
	static function radio($name,
		$objects,
		$selected_object = null,
		$default_selected = HTMLWidget::OPTION_NONE,
		$show_100 = true, 
		$text_100 = null,
		$show_any = false, 
		$text_any = null) {
	
		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}

		if (!$selected_object) {
			$selected = $default_selected;
		} elseif (is_object($selected_object)) {
			$selected = $selected_object->getID();
		} else {
			$selected = $selected_object;
		}

		$ret = '';
		if ($show_100) {
			$ret .= '<input type="radio" name="'.$name.'" value="'.HTMLWidget::OPTION_NONE.'"'.(($selected == HTMLWidget::OPTION_NONE) ? ' checked="checked"' : '' ).' />'.$text_100."<br />\n";
		}

		foreach ($objects as $object) {
			$id = $object->getID();
			$ret .= '<input type="radio" name="'.$name.'" value="'.$object->getID().'"'.(("$selected" == "$id") ? ' checked="checked" ' : '' ).' />'.$object->getName()."<br />\n";
			// Note the quotes above. It solves this:
			// http://bugs.php.net/bug.php?id=6766
		}

		if ($show_any) {
			$ret .= '<input type="radio" name="'.$name.'" value="'.HTMLWidget::OPTION_ANY.'" />'.$text_any."<br />\n";
		}

		return $ret;
	}


	/**
	 *	checkbox - build check boxes from the array of objects and selected item.
	 *	@param	string	The name of the widget.
	 *	@param	array	The array of objects, which must implement GFWidget, to be displayed.
	 *	@param	array	The optional array of id's of the selected objects.
	 *	@param	string	The id of the default selected item.
	 *	@param	boolean	Whether to show a 100 row.
	 *	@param	string	The text to show if you show a 100 row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 */
	static function checkbox($name,
		$objects,
		$selected_objects='',
		$selected_function_name='getId',
		$default_selected=100,
		$show_100=true,
		$text_100=null,
		$show_any=false,
		$text_any=null) {

		$ret='';
		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		if ($show_any) {
			$ret .= '
		<input type="checkbox" name="'.$name.'" value=""/> '.$text_any.'<br />';
		}
		if ($show_100) {
			$ret .= '
		<input type="checkbox" name="'.$name.'" value="100"';
			foreach ($selected_objects as $selected) {
				$ret .= (($selected==100) ? ' CHECKED' : '' );
			}
			$ret .= '/>'.$text_100.'<br />';
		}

		foreach ($objects as $object) {
			$ret .= '
		<input type="checkbox" name="'.$name.'" value="'.$object->getID().'"';

			foreach ($selected_objects as $selected) {
				$ret .= (($selected==$object->getID()) ? ' CHECKED' : '' );
			}
			$ret .= '/>'.$object->getName().'<br />';
		}

		return $ret;
	}

	/**
	 *	multiselect - build multiselect box from the array of objects and selected item.
	 *	@param	string	The name of the widget.
	 *	@param	array	The array of objects, which must implement GFWidget, to be displayed.
	 *	@param	array	The optional array of selected id's.
	 *	@param	string	The id of the default selected item.
	 *	@param	boolean	Whether to show a 100 row.
	 *	@param	string	The text to show if you show a 100 row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 */
	static function multiselect($name,
		$objects,
		$selected_objects=array(),
		$default_selected=100,
		$show_100=true,
		$text_100=null,
		$show_any=false,
		$text_any=null,
		$size=10) {

	//	$size=;

		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		$ret = '
		<select name="'.$name.'" multiple="multiple" size="'.$size.'">';

		if ($show_any) {
			$ret .= '
		<option value=""> '.$text_any.'</option>';
		}
		if ($show_100) {
			$ret .= '
		<option value="100"';
			foreach ($selected_objects as $selected) {
				$ret .= (($selected==100) ? ' selected="selected"' : '' );
			}
			$ret .= '>'.$text_100.'</option>';
		}

		foreach ($objects as $object) {
			$ret .= '
		<option value="'.$object->getID().'"';
			if (array_search($object->getID(), $selected_objects) !== false) {
				$ret .= ' selected="selected" ';
			}

/*			foreach ($selected_objects as $selected) {
				$ret .= (($selected==$object->getID()) ? ' selected="selected"' : '' );
			}
*/
			$ret .= '>'.$object->getName().'</option>';
		}
		$ret .= '
		</select>';

		return $ret;
	}
	
	/**
	 *	selectBoxFromAssoc - build a select box from the associative array.
	 *	@param	array	Associative array to build the select from.
	 *	@param	string	The name of the widget.
	 *	@param	string	The id of the object, which must implement GFWidget, that is selected.
	 *	@param	boolean	Whether to swap the keys/values of the associative array
	 *	@param	boolean	Whether to show a 'None' row.
	 *	@param	string	The text to show if you show a 'None' row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 */
	static function selectBoxFromAssoc (
		$arr,
		$select_name,
		$checked_val='',
		$swap=false,
		$show_100=true,
		$text_100=null,
		$show_any=false,
		$text_any=null) {
		if ($swap) {
			$keys=array_values($arr);
			$vals=array_keys($arr);
		} else {
			$vals=array_values($arr);
			$keys=array_keys($arr);
		}
		return self::selectBoxFromArrays ($keys,$vals,$select_name,$checked_val,$show_100,$text_100,$show_any,$text_any);
	}	

	/**
	 *	selectBoxFromArrays - build a select box from the arrays.
	 *	@param	array	keys (ids) to build the select box from.
	 *	@param	array	values (texts) to build the select box from.
	 *	@param	string	The name of the widget.
	 *	@param	string	The id of the element that is selected.
	 *	@param	boolean	Whether to show a 'None' row.
	 *	@param	string	The text to show if you show a 'None' row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 */
	static function selectBoxFromArrays (
		$vals,
		$texts,
		$select_name,
		$checked_val='',
		$show_100=true,
		$text_100=null,
		$show_any=false,
		$text_any=null) {
		$rows=count($vals);

		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		$return = '
			<select name="'.$select_name.'">';
		if ($show_any) {
			$return .= '
				<option value="">'.$text_any.'</option>';
		}
		if ($show_100) {
			$return .= '
				<option value="100"'.(( $checked_val==100 ) ? ' selected="selected"' : '' ).'>'.$text_100.'</option>';
		}
		for ($i=0; $i<$rows; $i++) {
			$return .= '
				<option value="'.$vals[$i].'"';
			if ($vals[$i] == $checked_val) {
				$return .= ' selected="selected"';
			}
			$return .= '>'.$texts[$i].'</option>';
		}

		$return .= '
			</select>';
		return $return;
	}

	/**
	 *	multiSelectFromAssoc - build a multipleselect box from the associative array.
	 *	@param	string	The name of the widget.
	 *	@param	array	Associative array to build the select from.
	 *	@param	string	The array of id's of the objects that are selected.
	 *	@param	string	The id of the object that is selected if no others are selected.
	 *	@param	boolean	Whether to show a 'None' row.
	 *	@param	string	The text to show if you show a 'None' row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 *	@param	string	The size (number of rows) of this multiselect box.
	 *	@param	boolean	Whether to swap the keys/values of the associative array
	 */
	static function multiSelectFromAssoc($name,
		$arr,
		$selected_objects=array(),
		$default_selected=100,
		$show_100=true,
		$text_100=null,
		$show_any=false,
		$text_any=null, 
		$size=10,
		$swap=false) {
		if ($swap) {
			$keys=array_values($arr);
			$vals=array_keys($arr);
		} else {
			$keys=array_keys($arr);
			$vals=array_values($arr);
		}
		return self::multiSelectFromArrays($keys, $vals, $name, $selected_objects, $default_selected, 
			$show_100, $text_100, $show_any, $text_any, $size);
	}

	/**
	 *	multiSelectFromAssoc - build a multipleselect box from the associative array.
	 *	@param	array	keys (ids) to build the select box from.
	 *	@param	array	values (texts) to build the select box from.
	 *	@param	string	The name of the widget.
	 *	@param	string	The array of id's of the objects that are selected.
	 *	@param	string	The id of the object that is selected if no others are selected.
	 *	@param	boolean	Whether to show a 'None' row.
	 *	@param	string	The text to show if you show a 'None' row.
	 *	@param	boolean	Whether to show a 'Any' row.
	 *	@param	string	The text to show if you show a 'Any' row.
	 *	@param	string	The size (number of rows) of this multiselect box.
	 */
	static function multiSelectFromArrays($keys, 
		$vals, 
		$name,
                $selected_objects=array(),
                $default_selected=100,
                $show_100=true,
                $text_100=null,
                $show_any=false,
                $text_any=null,
                $size=10) {

		$language = Language::getSessLang();

		if ($show_100 && is_null($text_100)) {
			$text_100 = $language->getText('Options.None');
		}
		if ($show_any && is_null($text_any)) {
			$text_any = $language->getText('Options.Any');
		}
		$ret = '
		<select name="'.$name.'" multiple="multiple" size="'.$size.'">';

		if ($show_any) {
			$ret .= '
		<option value=""> '.$text_any.'</option>';
		}
		if ($show_100) {
			$ret .= '
		<option value="100"';
			foreach ($selected_objects as $selected) {
				$ret .= (($selected==100) ? ' selected="selected"' : '' );
			}
			$ret .= '>'.$text_100.'<br />';
		}

		for ($i=0; $i<count($keys); $i++) {
			$ret .= '
		<option value="'.$keys[$i].'"';

			foreach ($selected_objects as $selected) {
				$ret .= (($selected==$keys[$i]) ? ' selected="selected"' : '' );
			}
			$ret .= '>'.$vals[$i].'</option>';
		}
		$ret .= '
		</select>';

		return $ret;
	}

	/**
	 *	multiSelectFromAssoc - build a multipleselect box from the associative array.
	 *	@param	array	nested folders array.
	 *	@param	array	call function with 0 value.
	 *	@param	string	call function with 0 value.
	 *	@param	string	call function with initially empty array.
	 *	@param	boolean	if true -> first element in tree is root, if false -> no root element.
	 */
	static function renderArranged($nested_folders,$folder,$spaces,$result,$root=false) {
		global $total_nest_count;
		$total_nest_count++;

		if (($root == true) && (sizeof($result) == 0)) {
			$result[0] =  "root";
			$spaces += 10;
		}

		if (isset($nested_folders[$folder]) && is_array($nested_folders[$folder])) {

			foreach ($nested_folders[$folder] as $folder) {

				$result[$folder->getId()] = str_repeat('&nbsp;',$spaces) . $folder->getName();
				if ( ($spaces-10) > 0) {
					$times = $spaces/10;
					$result[$folder->getId()] = '';
					for($i=0;$i<$times-1;$i++) {
						$result[$folder->getId()] .= '|' . str_repeat('&nbsp;',$spaces-(10*($times-1)));
					}
					$result[$folder->getId()] .= '|' . str_repeat('&nbsp;',10) . '|--'.$folder->getName();
					
				} elseif ( ($spaces-10) == 0) {
					$result[$folder->getId()] =  str_repeat('&nbsp;',$spaces-10) . '|' . str_repeat('&nbsp;',10) . '|--'.$folder->getName();
				}
				$result = HTMLWidget::renderArranged($nested_folders,$folder->getId(),$spaces+10,$result);
			}
			$spaces -= 10;
		}
		return $result;
	}	
	
	/**
	 *	radio - build a radiobutton list from the array of objects and selected item.
	 *	@param	string	The type of the widget.
	 */
	static function button($type ="", $text ="", $name ="", $id ="", $onclick ="", $enabled = true, $class="button") {
	
		static $buttonId = 0;


		if(strlen($type) == 0)	$type = "button";
		if(strlen($id) == 0){
			$buttonId ++;
			$id = "__button" . $buttonId . "__";			
		}
		
		$button = "";
		
		
		if($enabled){		
			
			$button .= "<a class='$class' onclick=\"document.getElementById('$id').click()\"><span>$text</span></a>";
			$button .= "<span style=\"display:none;\" class='$class'><input type='$type' class=''";
			
			if(sizeof($text) > 0)
				$button .= " value='$text'";
				
			if(sizeof($name) > 0)
				$button .= " name='$name'";	

			if(sizeof($id) > 0)
				$button .= " id='$id'";	

			if(sizeof($onclick) > 0)
				$button .= " onclick=\"$onclick\"";	
				
			$button .= "/></span>";										
		}else{
			$button  = " <del class='$class'><span> ";
			$button .= $text;	
			$button .= "</span></del>";	
		}
		
		return $button;					
	}				
}

?>
