<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 4415 2010-03-10 16:15:09Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/ActionManager.php');
require_once('html/Paginator.php');

abstract class Layout {

	private $browser; // browser object to determine the user's browser
	private $language; // language object
	protected $tableColCount;
	protected $breadCrumbs=array();
	protected $tableRowCounter;
	//protected $priorities=array(1=>'ff3131',2=>'d75064',3=>'b07098',4=>'8890cc',5=>'61afff');
	//protected $priorities=array(1=>'ff3131',2=>'c6642a',3=>'8e9622',4=>'56c81b',5=>'1ef914');
	protected $priorities=array(1=>'#ff3131',2=>'#ff5b3f',3=>'#ff884d',4=>'#ffad5b',5=>'#ffd668');

	/**
	 * Layout() - Constructor
	 */
	function __construct() {
		$this->language = Session::getSession()->getLanguage();
		
	}
	function getPriorityColor($priority) {
		if (array_key_exists($priority,$this->priorities)) {
			return $this->priorities[$priority];
		} else {
			return '#ff884d'; // return priority color 3 as default
		}
	}

	function getHelpSection() {
		$arr=explode('/',$_SERVER['REQUEST_URI']);
		//project url space
		$str='';
		if (isset($arr[2])) {
			switch ($arr[2]) {
				case 'project': {
					$str='project';
					if (isset($arr[4])) {
						switch ($arr[4]) {
							case 'tracker':
							case 'reporting':
							case 'search':
							case 'docman':
							case 'news':
							case 'frs':
							case 'wiki':
							case 'mailman':
							case 'forum':
							case 'cruisecontrol':
							case 'build':
							case 'scmcvs':
							case 'scmsvn':
							case 'scmgit':
							case 'scmcc':
							case 'scmvss':
							case 'scmperforce': {
								$str .= '-'.$arr[4];
								if (isset($arr[5]) && $arr[5]=='admin') {
									$str .= '-admin';
								}
								break;
							}
							case 'admin': {
								$str .= '-admin';
								break;
							}
						}
					}
					break;
				}
				case 'admin': {
					return 'site-admin';
					break;
				}
			}
		}
		return $str;
	}

	function getQuickJump($table_mode) {
		if (Session::getSession()->isLoggedIn()) {
			$lang = Session::getSession()->getLanguage();
			if (isset($table_mode) && $table_mode == true) {
			$return = 	"<table class='quickJumpMenu' summary=''>" . 
					"<tr><td class='quickJumpMenu'><a class='quickJumpMenu' href=\"/gf/register/?action=ProjectAdd\">" . $lang->getText('QuickJump.AddProject') . "</a></td></tr>" .
					"<tr><td class='quickJumpMenu'><a class='quickJumpMenu' href=\"/gf/my/reports/\">" . $lang->getText('QuickJump.Reports') . "</a></td></tr>" .
					"<tr><td class='quickJumpMenu'><a class='quickJumpMenu' href=\"/gf/my/\">" . $lang->getText('QuickJump.MyStuff') . "</a></td></tr>" .
					"<tr><td class='quickJumpMenu'><a class='quickJumpMenu' href=\"/gf/my/dashboard/\">" . $lang->getText('QuickJump.Dashboard') . "</a></td></tr>" .
					"<tr><td class='quickJumpMenu'><a class='quickJumpMenu' href=\"/gf/my/activity/\">" . $lang->getText('QuickJump.Activity') . "</a></td></tr>" .
					"<tr><td class='quickJumpMenu'><a class='quickJumpMenu' href=\"/gf/my/gantt/\">" . $lang->getText('QuickJump.Gantt') . "</a></td></tr>" .
					"<tr><td class='quickJumpMenu'><a class='quickJumpMenu' href=\"/gf/my/?action=LatestForumMessages\">" . $lang->getText('QuickJump.Forums') . "</a></td></tr>";
			} else {
				$return = '<form name="quicknavform" action="">
				<select name="quicknav" onchange="location.href=document.quicknavform.quicknav.value">
				<option value="">'.$lang->getText('QuickJump.JumpTo').'</option>
				<option value="/gf/register/?action=ProjectAdd">&nbsp;&nbsp;'.$lang->getText('QuickJump.AddProject').'</option>
				<option value="/gf/my/">'.$lang->getText('QuickJump.MyStuff').'</option>
				<option value="/gf/my/reports/">&nbsp;&nbsp;'.$lang->getText('QuickJump.Reports').'</option>
				<option value="/gf/my/dashboard/">&nbsp;&nbsp;'.$lang->getText('QuickJump.Dashboard').'</option>
				<option value="/gf/my/activity/">&nbsp;&nbsp;'.$lang->getText('QuickJump.Activity').'</option>
				<option value="/gf/my/gantt/">&nbsp;&nbsp;'.$lang->getText('QuickJump.Gantt').'</option>
				<option value="/gf/my/?action=LatestForumMessages">&nbsp;&nbsp;'.$lang->getText('QuickJump.Forums').'</option>
	';
			}
			$ids=Session::getSession()->getUser()->getAllProjectIds();
			if (count($ids) > 0) {
				$sql="SELECT pr.unix_name,pr.project_name,pl.plugin_name 
				FROM project pr, project_plugin pp, plugin pl
				WHERE pr.project_id IN (".implode(',',$ids).")
				AND pr.project_id=pp.project_id AND
				pp.plugin_id=pl.plugin_id
				ORDER BY pr.unix_name";
				$con=Propel::getConnection();
				$stmt=$con->createStatement();
				$rs=$stmt->executeQuery($sql);
				$last_project='';
				while ($rs->next()) {
					$plugin_name = $rs->getString('plugin_name');
					// Ugly hack -- Jabber plugin shouldn't be shown since it doesn't have a main page
					if ($plugin_name == 'projectjabber') {
						continue;
					}
					$projname=$rs->getString('unix_name');
					if ($projname != $last_project) {
						$last_project=$projname;
						if (isset($table_mode) && $table_mode == true) {
							$return .= '<tr><td class=\'quickJumpMenu\'><a class=\'quickJumpMenu\' href="/gf/project/'.$projname.'/">'.$rs->getString('project_name').'</a></td></tr>';
						} else {
							$return .= '			<option value="/gf/project/'.$projname.'/">'.$rs->getString('project_name').'</option>';
						}
					}
					if (isset($table_mode) && $table_mode == true) {
						$return .= '<tr><td class=\'quickJumpMenu\'><a class=\'quickJumpMenu\' href="/gf/project/'.$projname.'/'.$rs->getString('plugin_name').'/">&nbsp;&nbsp;'.ucwords($rs->getString('plugin_name')).'</a></td></tr>';
					} else {
						$return .= '			<option value="/gf/project/'.$projname.'/'.$rs->getString('plugin_name').'/">&nbsp;&nbsp;'.ucwords($rs->getString('plugin_name')).'</option>';
					}
				}
			}
			if (isset($table_mode) && $table_mode == true) {
				$return .= '</select></form>';
			} else {
				$return .= '</table></div>';
			}
			return $return;
		} else {
			return '';
		}
	}

	function addBreadCrumb($link) {
		$this->breadCrumbs[]=$link;
	}
	/**
	 * Return a list with the extra stylesheets to use. In the base class, we don't use any
	 *
	 */
	function getExtraCSS() {
		return array();
	}

	function headerStart($params) {
		if (!isset($params['title']) || $params['title'] == '') {
			$title = GFConf::get('system.name');
		} else {
			$title = GFConf::get('system.name') .' > ' . $params['title'];
		}
	//	print '<?xml version="1.0" encoding="' . $this->language->getEncoding(). '"? >';
	
		// Extra things to add in <head>
		if (isset($params['extra_head']) && is_array($params['extra_head'])) {
			$extra_heads = '';
			foreach ($params['extra_head'] as $extra_head) {
				$extra_heads .= $extra_head."\n";
			}
		} else {
			$extra_heads = '';
		}
		
		$extra_css = $this->getExtraCSS();
		
		?>

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="<?php echo $this->language->getLanguageCode(); ?>">

  <head>
	<meta http-equiv="Content-Type" content="text/html; charset=<?php echo $this->language->getEncoding(); ?>" />
	
	<?php echo $extra_heads; ?>
	
	<title><?php echo $title; ?></title>
<?php
	$links_arr = $this->generateRSSLinks();
	if (count($links_arr) > 0) {
		foreach ($links_arr as $values) {
?>
	<link rel="alternate" type="application/rss+xml" title="<?php echo $values['title'] ?>" href="<?php echo $values['url'] ?>" />
<?php
		}
	}
	//make persistent tabber tabs
	if (isset($GLOBALS['tabbertabsetcookie'])) {
		include('lib/external/tabberpersistent.php');
	}
	//include javascript and css files directly - may greatly increase speed on SSL + firefox
	if (GFConf::get('system.include_js_css')) {
		?>
<style type="text/css">
<!-- 
<?php
		include('www/themes/'.$this->getName().'/GFTheme.css');
		include('www/themes/'.$this->getName().'/viewvc.css');
		include('www/themes/'.$this->getName().'/textwiki.css');
?>
-->
</style>
<script type="text/javascript">
<!--
<?php
		include('www/js/gforge5.js');
		include('www/js/tooltip.js');
		include('www/tabber.js');
		include('www/jquery.js');
		include('www/ddaccordion.js');
		include('www/mktree.js');
		include('www/prototype.js');
?>

-->
</script>
<?php
		// The extra_css are given as URLs so we must include them using the <link> tag
		foreach ($extra_css as $css) {
			echo '<link href="'.$css.'" rel="stylesheet" type="text/css" />'."\n";
		}

	} else {
	//don't include CSS and JS files directly in the page - makes a smaller page
?>
	<link href="/themes/<?php echo $this->getName(); ?>/GFTheme.css" rel="stylesheet" type="text/css" />
	<link href="/themes/<?php echo $this->getName(); ?>/viewvc.css" rel="stylesheet" type="text/css" />
	<link href="/themes/<?php echo $this->getName(); ?>/textwiki.css" rel="stylesheet" type="text/css" />
	<script type="text/javascript" src="/js/gforge5.js"></script>
	<script type="text/javascript" src="/js/tooltip.js"></script>
	<script type="text/javascript" src="/tabber.js"></script>
	<script type="text/javascript" src="/jquery.js"></script>
	<script type="text/javascript" src="/ddaccordion.js"></script>
	<script type="text/javascript" src="/mktree.js"></script>
	<script type="text/javascript" src="/prototype.js"></script>
<?php
		foreach ($extra_css as $css) {
			echo '<link href="'.$css.'" rel="stylesheet" type="text/css" />'."\n";
		}
	} //end include_js_css
?>

<script type="text/javascript">
ddaccordion.init({
	headerclass: "expandable", //Shared CSS class name of headers group
	contentclass: "submenu", //Shared CSS class name of contents group
	collapseprev: true, //Collapse previous content (so only one open at any time)? true/false 
	defaultexpanded: [], //index of content(s) open by default [index1, index2, etc] [] denotes no content
	animatedefault: false, //Should contents open by default be animated into view?
	persiststate: false, //persist state of opened contents within browser session?
	toggleclass: ["", "openheader"], //Two CSS classes to be applied to the header when it's collapsed and expanded, respectively ["class1", "class2"]
	togglehtml: ["suffix", "", ""], //Additional HTML added to the header when it's collapsed and expanded, respectively  ["position", "html1", "html2"] (see docs)
	animatespeed: "normal" //speed of animation: "fast", "normal", or "slow"
})

oldload = window.onload;
function expandableExpandItem(){};
function docmanExpandTree(){};
window.onload=function(){oldload();convertTrees();window.setTimeout('expandableExpandItem()',100);docmanExpandTree();}
</script>

</head>
<?php
	}

	/**
	 *	header() - "steel theme" top of page
	 *
	 * @param	array	Header parameters array
	 */
	function header($params) {
		if (isset($GLOBALS['headershown']) && $GLOBALS['headershown']) {
			return;
		} else {
			$GLOBALS['headershown']=true;
		}
		$this->headerStart($params);

	}

	function footerEnd($params) {
?>

</body>
</html>
<?php

	}

	function footer($params=array()) {

		//footer code

		$this->footerEnd($params);
	}

	function getRootIndex() {
		return $this->rootindex;
	}

	//returns a complex array of items. Each template will override this and add additional elements
	//like tracker elements or forum elements
	/**
	 *      getPluginNavElements - this overrides parent class and adds its own
	 *      plugin listener
	 *      @return PluginResultSet
	 */
	function getPluginNavElements($params) {
		return ActionManager::Notify('GET_LOCAL_NAV_ELEMENTS',$params);
/*
	should return an array like this:

	array(
		['title']='Title';
		['elements']= array(
		array('link'=>'/path/to/file/?action',
			'description'='Title'),
		array('link'=>'/path/to/file/?action',
			'description'='Title'),
		array('link'=>'/path/to/file/?action',
			'description'='Title')
		)
	}
*/
	}

	/**
	 * getTemplateNavElements - this function can be overridden
	 * by subclasses so you can add navigation entries
	 * @return array - complex array of data.
	 */
	function getTemplateNavElements($params) {
		$pine = $this->getPluginNavElements($params);
		$ret=array();
		foreach ($pine as $result) {
			$ret[]=$result->getResult();
		}
		return $ret;
	}

	function renderLocalNavElements($params) {
		$arr = $this->getTemplateNavElements($params);
		if (count($arr) == 0) {
			// return empty string if we don't have to show a nav bar
			return '';
		}
		
		$nav = '';

		foreach ($arr as $res) {

			$nav .= '
	<strong>'.$res['title'].'</strong>
		<ul>';
			foreach ($res['elements'] AS $el) {
				$nav .= '
	<li><a href="'.$el['link'].'">'.$el['description'].'</a></li>';
			}
			$nav .= '
		</ul>';
		}
		
		return $nav;
	}

	/**
	 * infoTableTop() - Top HTML box
	 *
	 * @param   string  Box title
	 * @param   bool	Whether to echo or return the results
	 * @param   string  The box background color
	 */
	function infoTableTop($title) {
		return '
		<!-- Box Top Start -->

		<table cellspacing="0" cellpadding="0" width="100%" border="1">
		<tr><td>

		<!-- Box Top End -->';
	}

	/**
	 * infoTableMiddle() - Middle HTML box
	 *
	 * @param   string  Box title
	 * @param   string  The box background color
	 */
	function infoTableMiddle($title) {
		return '
		<!-- Box Middle Start -->

		</td></tr>
		<tr><td>

		<!-- Box Middle End -->';
	}

	/**
	 * infoTableBottom() - Bottom HTML box
	 *
	 * @param   bool	Whether to echo or return the results
	 */
	function infoTableBottom() {
		return '
			<!-- Box Bottom Start -->
		</td></tr>
		</table><br />
		<!-- Box Bottom End -->';
	}

	/**
	 * getTableRowStyle() - Get an alternating row style for tables
	 *
	 * @param			   int			 Row number
	 */
	function getTableRowStyle($i) {
		if ($i % 2 == 0) {
			return 'bgcolor="#EAEAEA"';
		} else {
			return 'bgcolor="#E0E0E0"';
		}
	}

	/**
	 * tableTop() - Takes an array of titles and builds the first row of a new table.
	 *
	 * @param	   array   The array of titles
	 * @param	   array   The array of title links
	 */
	function tableTop ($title_arr,$links_arr=false) {
		$this->tableRowCounter=0;
		$return = '
		<table cellspacing="1" cellpadding="1" width="80%" border="0" class="highlightTable">
			<tr>';

		$count=count($title_arr);
		if ($links_arr) {
			for ($i=0; $i<$count; $i++) {
				$return .= '
				<th align="center"><a class="sortbutton" href="'.$links_arr[$i].'">'.$title_arr[$i].'</a></th>';
			}
		} else {
			for ($i=0; $i<$count; $i++) {
				$return .= '
				<th class="tableTitle" align="center">'.$title_arr[$i].'</th>';
			}
		}
		return $return.'</tr>';
	}

	/**
	 * tableRow build a <tr><td> set for this array
	 * @param	array	
	 */
	function tableRow($array,$array2=array()) {
		$return = '
			<tr '.$this->getTableRowStyle($this->tableRowCounter++).'>';
		for ($i=0; $i<count($array); $i++) {
			if (isset($array2[$i])) {
				$str=$array2[$i];
			} else {
				$str='';
			}
			$return .= '<td '.$str.'>'.$array[$i].'</td>';
		}
		$return .= '
			</tr>';
		return $return;
	}
	
	/**
	 * Builds a row that spans the entire table
	 */
	function tableFullRow($text) {
		$return = '<tr '.$this->getTableRowStyle($this->tableRowCounter++).'>';
		$return .= '<td colspan="'.$this->tableColCount.'">'.$text.'</td>';
		$return .= '
			</tr>';
		return $return;
	}
	
	/**
	 * Displays a row with a paginator
	 */
	function listPaginatorRow(Paginator $paginator) {
		$paginator_text = $paginator->getPaginator();
		if (strlen($paginator_text) == 0) {
			// no need to display the paginator
			return '';
		} else {
			$text = '<div align="center">'.$paginator->getPaginator().'</div>';
			return $this->tableFullRow($text);
		}
	}
	
	function tableBottom($button=array()) {
		$return = '';
		if (count($button) > 0) {
			$keys=array_keys($button);
			$vals=array_values($button);
			$rowelements=array();
			for ($i=0; $i<$this->tableColCount-count($button); $i++) {
				$rowelements[]='&nbsp;';
			}
			foreach ($button as $href => $text) {
				$rowelements[]='<a href="'.$href.'">'.$text.'</a>';
			}
			$return .= $this->tableRow($rowelements);
		}
		$return .= '
			</table>';
		return $return;
	}

	function topTabs($params) {
//		echo $this->renderTabs($TABS_DIRS,$TABS_TITLES,false,$selected,$this->COLOR_SELECTED_TAB,'100%');

		if (array_key_exists('selectedtoptab', $params)) {
			$selectedTab = $params['selectedtoptab'];
		} else {
			$selectedTab = false;
		}
		
		$tabs = array();
		// "Main page" tab
		$tabs[] = array(
				'text'	=>	Language::getSessText('Tabs.Home'),
				'id'	=>	'home',
				'action'	=>	'/gf/'
			);
			
		// "My page" tab
		$tabs[] = array(
				'text'	=>	Language::getSessText('Tabs.MyPage'),
				'id'	=>	'my',
				'action'	=>	'/gf/my/'
			);			

		if (GFConf::get('system.usersTabAvailable') == true) {	
			$tabs[] = array(
					'text'	=>	Language::getSessText('Tabs.Users'),
					'id'	=>	'user',
					'action'	=> 	'/gf/user/'
				);
		}

/*
//EXPRESS
		// Search page tab
		$tabs[] = array(
				'text'	=>	Language::getSessText('Tabs.Search'),
				'id'	=>	'search',
				'action'	=> 	'/gf/search/'
			);
*/
		$tabs[] = array(
				'text'	=>	Language::getSessText('Tabs.Projects'),
				'id'	=>	'project',
				'action'	=> 	'/gf/project/'
			);

		if (Session::getSession()->isSiteAdmin()) {
			// "Admin" tab
			$tabs[] = array(
					'text'	=>	Language::getSessText('Tabs.Admin'),
					'id'	=>	'admin',
					'action'	=>	'/gf/admin/'
				);			
		}

		$prs = ActionManager::notify("BUILD_MAIN_TABS");
		foreach ($prs as $result) {
			$tabs[] = $result->getResult();
		}
		
/*		// if there's a project, build a project tab at the end
		if (isset($params['project'])) {
			$project = $params['project'];
			$tabs[] = array(
				'text'	=>	$project->getProjectName(),
				'id'	=>	'current_project',
				'action'	=> 	GFUrl::getUrl()->projectURL($project)
				);
			
			$selectedTab = 'current_project';
		}
*/

		echo $this->renderTabs($tabs, $selectedTab, false, $params);
	}

	function breadCrumbs($title) {
		$return = '<a href="/">'.Language::getSessText('Tabs.Home').'</a>';
		$arr = $this->breadCrumbs;
		for ($i=(count($arr)-1); $i>=0; $i--) {
			$return .= ' » '.$arr[$i];
		}
		$tarr=explode(' > ',$title);
		$return .= ' » '.$tarr[count($tarr)-1];
		return $return;
	}

	/**
	 *	secondTabs() - Prints out the project tabs, contained here in case
	 *		we want to allow it to be overriden
	 *
	 *	@param	array	project, selected tab
	 */
	function secondTabs($params) {

//		echo $this->renderTabs($TABS_DIRS,$TABS_TITLES,true,$selected,'white','100%');

		if (array_key_exists('selectedbottomtab', $params)) {
			$selectedTab = $params['selectedbottomtab'];
		} else {
			$selectedTab = false;
		}
		
		$project = GFUrl::getUrl()->getProject();
		$projectbaseurl='/gf/project/'.$project->getUnixName().'/';
		$tabs = array();
		// "Summary" tab
		$tabs[] = array(
				'text'	=>	Language::getSessText('Tabs.Summary'),
				'id'	=>	'summary',
				'action'	=>	$projectbaseurl
			);
		// "Admin" tab
		if ($project->userIsAdmin()) {
			$tabs[] = array(
					'text'	=>	Language::getSessText('Tabs.ProjectAdmin'),
					'id'	=>	'project_admin',
					'action'	=>	$projectbaseurl.'admin/'
				);
		}
		// "Reporting" tab
		$tabs[] = array(
				'text'	=>	Language::getSessText('Tabs.ProjectReporting'),
				'id'	=>	'reporting',
				'action'	=>	$projectbaseurl.'reporting/'
			);
		// "Search" tab
		$tabs[] = array(
				'text'	=>	Language::getSessText('Tabs.Search'),
				'id'	=>	'search',
				'action'	=>	$projectbaseurl.'search/'
			);		
		$prs = ActionManager::notify("BUILD_PROJECT_TABS",$params);
		foreach ($prs as $result) {
			$val = $result->getResult();
			if (array_key_exists(0, $val)) {
				// Plugin returned several tabs
				foreach ($val as $tab) {
					$tabs[] = $tab;
				}
			} else {
				// Plugin returned a single tab
				$tabs[] = $result->getResult();
			}
		}
		
		echo $this->renderTabs($tabs, $selectedTab, true, $params);
	}


	protected function renderTabs($tabs, $selectedTab, $nested) {

		foreach ($tabs as $tab) {
			if ($tab['id'] == $selectedTab) {
				$return .= '&nbsp;<strong>[&nbsp;<a href="'.$tab['action'].'">'.$tab['text'].'</a>&nbsp;]</strong>&nbsp;';
			} else {
				$return .= '&nbsp;[&nbsp;<a href="'.$tab['action'].'">'.$tab['text'].'</a>&nbsp;]&nbsp;';
			}
		}
		return $return;
	}

	function renderSearchBox($params=array()) {
		$language = Language::getSessLang();
		if ( (isset($params['project'])) && (is_object($params['project']))) {
			echo '<form style="display: inline;" action="/gf/project/' . $params['project']->getUnixName() . '/search/" method="get">';
			echo '<input type="hidden" name="project_id" value="' . $params['project']->getProjectId() . '"/>';
		} else {
			echo '<form style="display: inline;" action="/gf/search/" method="get">';
		}
		echo '<select  name="search">';
		$prs = ActionManager::notify('BUILD_SEARCH_COMBO',$params);
		//$something_checked = false;
		foreach ($prs as $result) {
			$options = $result->getResult();
			foreach ($options as $option) {
				if (array_key_exists('checked',$option)) {
					$checked = " selected ";
					//$something_checked = true;
				} else {
					$checked = " ";
				}
				echo '<option' . $checked . 'value="' . $option['value'] . '"/>' . $option['display'];
			}
		}

		if (array_key_exists('section',$params)) {
			switch ($params['section']) {
				case 'project' : {
				echo '<option selected="project_0" value="project_0">'.Language::getSessText('Search.SearchProjects').'</option>'; // select this as default
				echo '<option value="user_0">'.Language::getSessText('Search.SearchUsers').'</option>';
				break;
				}
				case 'user' : {
					echo '<option selected="user_0" value="user_0">'.Language::getSessText('Search.SearchUsers').'</option>'; 
					echo '<option value="project_0">'.Language::getSessText('Search.SearchProjects').'</option>'; // select this as default
					break;
				}			
			}
		} else {
			echo '<option value="project_0">'.Language::getSessText('Search.SearchProjects').'</option>';
			echo '<option value="user_0">'.Language::getSessText('Search.SearchUsers').'</option>'; 			
		}
		
		echo '</select>&nbsp;<input type="text" size="12" name="words" value="" />&nbsp;';

		echo HTMLWidget::button("submit",$language->getSessText('SearchBox.search'),"Search","","") ;
		echo '</form>';

	}
	
	/**
	 * Returns the URL of an image for the current theme
	 * @param	string $imageName
	 * @param	int $widht	Width, zero if unknown
	 * @param	int $height	Height, zero if unknown
	 * @param	array $extra	Associative array with extra parameters
	 */
	public function image($imageName, $width=0, $height=0, $extra=array()) {
		$imgURL = $this->getImgRoot().'/'.$imageName;
		$tag = '<img src="'.$imgURL.'" ';
		if ($width) {
			$tag .= 'width="'.$width.'" ';
		}
		if ($height) {
			$tag .= 'height="'.$height.'" ';
		}
		$alt = false;
		foreach ($extra as $name => $value) {
			if (!empty($name)) {
				$tag .= $name.'="'.$value.'" ';
				if ($name=='alt') {
					$alt = true;
				}
			}
		}
		if (!$alt) {
			$tag .= 'alt = "alternative text"'; // this attribute is a must in xhtml. it should be passed as argument
										// but if not, at least we don't want the page broken
		}
		$tag .= '/>';
		return $tag;
	}
	
	public function getImgRoot() {
		return GFUrl::getUrl()->buildURL('/themes/'.$this->getName().'/images');
	}
	
	public function getImgUrl($image_path) {
		return $this->getImgRoot().'/'.$image_path;
	}

	public function getName() {
		return 'default';
	}

	public function generateRSSLinks() {
		$url = GFUrl::getUrl();
		$links_arr = array();
		if (!is_object(GFUrl::getUrl()->getPlugin()) && !is_object(GFUrl::getUrl()->getProject()) && (GFUrl::getUrl()->getSubdir() == '')) {
			$links_arr['sitenews']['title'] = GFConf::get('system.name').' - News';
			$links_arr['sitenews']['url'] = $url->getFullURL($url->baseURL('Rss', '', '', array('section' => 'sitenews')));
	
			$links_arr['siteposts']['title'] = GFConf::get('system.name').' - Posts';
			$links_arr['siteposts']['url'] = $url->getFullURL($url->baseURL('Rss', '', '', array('section' => 'siteposts')));
		}
		return $links_arr;
	}

}
?>
