<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version $Id$
 * @author Daniel Perez	danielperez.arg@gmail.com
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('notification/GFNotification.php');
require_once("plugin/ActionManager.php");
require_once('StringUtils.php');
require_once('GFUrl.php');

/**
 * Note that the "Forum message notification" is actually a Forum notification. When
 * ANY message is added to the forum, we must notify the user
 */
class ForumMessageNotification implements GFNotification {
	private $ForumMessage;
	private $Forum;
	private $users;

	// This string is used when sending the notification mail for identifying the user response
	const FORUM_MAIL_MARKER = '#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+#+';

	function __construct(ForumMessage $ForumMessage) {
		$this->ForumMessage = $ForumMessage;
		$this->Forum = $ForumMessage->getForumThread()->getForum();
	}

	/**
	 *	getNotificationMessage - construct a string of this message content.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationMessage(User $User) {
		return $this->getGenericMessage();
	}
	
	/**
	 * This "generic" message will be used for both users and non-users when
	 * a new post is made
	 * @return	string
	 */
	private function getGenericMessage() {
		$url = GFurl::getUrl();
		
		if ($this->Forum->getSection() != 'project') {
			throw new Exception('Tried to notify about a non-project forum');
		}
		
		$project = ProjectPeer::retrieveByPk($this->Forum->getRefId());

		// make the body
		$body = "Thread " . $this->ForumMessage->getForumThread()->getThreadName()  ." had a new message posted at ". $this->ForumMessage->getPostDate() . 
			"\nYou can respond by visiting: ".
			"\n" . $url->getFullURL($url->projectUrl($project,'ForumBrowse', 'forum', '', array('_forum_action'=>'MessageReply','message_id' => $this->ForumMessage->getForumMessageId())),true) .
			"\nOr by replying to this e-mail entering your response between the following markers: ".
			"\n".
			"\n".self::FORUM_MAIL_MARKER.
			"\n(enter your response here)".
			"\n".self::FORUM_MAIL_MARKER.
			"\n\n". 
			"Submitted By: ". $this->ForumMessage->getUser()->getFirstName() . " " . $this->ForumMessage->getUser()->getLastName() .
			" (". $this->ForumMessage->getUser()->getUnixName() . ")"."\n\n";

		$body .= StringUtils::getPlainUTF8FromHtml($this->ForumMessage->getBody()) ." \n";
		$body .= "\n\n______________________________________________________________________".
		"\nYou are receiving this email because you elected to monitor this forum.".
		"\nTo stop monitoring this forum, login to ". $url->getFullUrl($url->baseUrl('Login', '', 'account')) ." and visit: ".
		"\n".$url->getFullUrl($url->getMonitorUrl('forum', $this->Forum->getPrimaryKey(), $url->projectUrl($project, '', 'forum'), false),true);

		return $body;
	}

	/**
	 *	getNotificationSubject - construct a string of this message subject.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationSubject(User $User) {
		return $this->getGenericSubject();
	}

	/**
	 * This "generic" subject will be used for both users and non-users when
	 * a new post is made
	 * @return	string
	 */
	private function getGenericSubject() {
		$subject = "[".$this->Forum->getForumName()."] ". StringUtils::unHtmlSpecialChars($this->ForumMessage->getSubject());
		return $subject;
	}


	/**
	 *	getUsers - get an array of user objects
	 *	@return array
	 */
	public function getUsers() {
		return $this->ForumMessage->getForumThread()->getForum()->getMonitoringUsers(); // thread or forum or project monitors
	}

	/**
	 *	getSection - retrieves the section name of the forum message
	 *	@return string
	 */	
	function getSection() {
		return ForumMessage::SECTION;
	}
	
	/**
	 *	getRefId - retrieves the id of the forum message that the notification belongs to
	 *	@return int
	 */	
	function getRefId() {
		return $this->ForumMessage->getForumMessageId();
	}
	
	/**
	 * This will notify of the new message to the addresses defined in 
	 * "send_all_posts_to" field
	 */
	public function notifyAllPosts() {
		$addresses = StringUtils::getAddressesFromList($this->Forum->getSendAllPostsTo());
		
		foreach ($addresses as $address) {
			$message = $this->getGenericMessage();
			$subject = $this->getGenericSubject().' (forummessage-'.$this->ForumMessage->getPrimaryKey().')';
			
			SystemUtils::sendMail($subject, $message, $address,false,SystemUtils::getFromAddress('gforge-gateway'));
		}
	}
}

?>
