<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once("system/Cronjob.php");
require_once('core/CronHistory.php');
require_once('core/ProjectPluginPeer.php');
require_once('core/DeletedMailingListPeer.php');
require_once('core/PluginPeer.php');
require_once('plugin/ActionManager.php');
require_once('plugins/mailman/lib/MailmanPeer.php');

class CreateListsCronjob extends Cronjob {
	
	public function __construct() {
		$this->jobname = basename(__FILE__);
		parent::__construct();
	}

	public function run() {			
		//create a list of active mailing lists
		$act_lists=array();
			
		$this->logCron("Checking active lists");
		$ll_ha = popen (GFConf::get('mailman.mailmanDir')."/bin/list_lists","r");
		
		while (!feof($ll_ha)) {
			$ll_out = fgets($ll_ha, 4096);
			if (stristr($ll_out,"matching mailing lists") !== FALSE) {
				continue;
			}			
			if (stristr($ll_out,"no matching mailing lists found") !== FALSE) {
				$this->logCron("No mailing lists: create Mailman list first");
				$this->commitLog();
				return false;
			}			
			
			$list_desc = explode(" ",trim($ll_out));				
			$act_lists[] = strtolower($list_desc[0]);	
			
		}
		
		$this->logCron('Existing mailing lists: '.implode(', ', $act_lists));

		pclose($ll_ha);
		
		//create mailing lists
		$arr = MailmanPeer::doSelectJoinUser(new Criteria());
		
		$lists = array();

		foreach ($arr as $item) {
			$list_name = strtolower($item->getListName());	// Must be lowercase

			if (!in_array($list_name,$act_lists)) { //new list
				$this->logCron("Creating Mailing List: ".$list_name);
				$cmd = escapeshellcmd(GFConf::get('mailman.mailmanDir')."/bin/newlist -q ".$list_name." ".$item->getUser()->getEmail()." ".$item->getListPassword());
				//echo "\n Command: $cmd";
				$output = array();
				$failed = SystemUtils::execute($cmd, $output);
				
				if($failed) {
					$this->logCron('Failed to create '.$list_name.", skipping");
					continue;
				}			
				//set list active
				//$obj = MailmanPeer::retrieveByPK($item->getMailmanId()); 
				$item->setStatus(Mailman::STATUS_CREATED);
				$item->save();  
				
				// send the password to the creator of the list
				SystemUtils::sendMail(GFConf::get('system.name') . ' - Password for mailing list ' . $item->getListName(),'You are receiving this email because you created the '.$item->getListName().' mailing list on '.GFConf::get('system.hostName')."\n".
				'The password for the list is : ' . $item->getListPassword() . " \n Please be careful at handling the password and do not give it to anyone.",UserPeer::retrieveByPk($item->getCreatedBy())->getEmail());
				//privatize 
				$this->logCron("Privatizing ".$list_name);
				$output = array();
				$command = escapeshellcmd(GFConf::get('mailman.mailmanDir').'/bin/config_list -i '.dirname(__FILE__).'/privatize_list.py '.$list_name).' 2>&1';
				$privatizeFailed = SystemUtils::execute($command, $output);

			} else { //act list
				if ($item->getStatus() == Mailman::STATUS_REQUESTED) {
					$item->setStatus(Mailman::STATUS_CREATED);
					$item->save();
				}
				if(!$item->getIsPublic()) {
					//privatize 
					$this->logCron("Privatizing ".$list_name);
					$output = array();
					$command = escapeshellcmd(GFConf::get('mailman.mailmanDir').'/bin/config_list -i '.dirname(__FILE__).'/privatize_list.py '.$list_name).' 2>&1';
					$privatizeFailed = SystemUtils::execute($command, $output);
				}
			}
		}
		
		
		//delete mailing lists
		$c = new Criteria();
//		$c->add(DeletedMailingListPeer::ISDELETED, Mailman::STATUS_DELETE_REQUEST);		
		$arr = DeletedMailingListPeer::doSelect($c);
		
		foreach ($arr as $item) {
			passthru(escapeshellcmd(GFConf::get('mailman.mailmanDir')."/bin/rmlist -a ".$item->getMailingListName()), $failed);
			if($failed){
				$this->logCron("Could not remove the list ".$item->getMailingListName());
			} else {
				$this->logCron("Deleted mailing list: ".$item->getMailingListName());
				$item->delete();
			}
		}
		$this->commitLog();
	}
}

$cronjob = new CreateListsCronjob();
$cronjob->run();
?>
