<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Group.class,v 1.94 2005/09/06 18:13:29 danper Exp $
 * @author Tim Perdue tim@gforge.org
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('notification/GFNotification.php');
require_once('StringUtils.php');

class NewsNotification implements GFNotification {
	protected $news;
	protected $project;

	function __construct(News $news) {
		$this->news = $news;
		$this->project = $news->getProject();
	}

	/**
	 *	getNotificationMessage - construct a string of this message content.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationMessage(User $User) {
		$url = GFurl::getUrl();
		// make the body
		$body = StringUtils::getPlainUTF8FromHtml($this->getDetails($User));
		
		$body .= "\n";
		$body .= "\n";
		$body .= 'Read more at '.$url->getFullURL($url->projectUrl($this->news->getProject(), '', 'news'), true);

		$body .= "\n\n______________________________________________________________________".
				"\n\nYou received this email because you ".
				"\nrequested to be notified when new versions of this package ".
				"\nwere released. If you don't wish to be notified in the ".
				"\nfuture, please login to " . $url->getFullUrl($url->baseUrl('Login', '', 'account'),true) . " and click this link: ".
				"\n" . $url->getFullUrl($url->getMonitorURL(News::SECTION, $this->project->getPrimaryKey(), $url->projectUrl($this->project, '', 'news'), false),true);

		return $body;
		
	}

	/**
	 *	getSummary - construct a string of this message summary (html chars unconverted).
	 *
	 *	@param	User
	 *	@return string
	 */
	function getSummary(User $User) {

		$summary = $this->news->getSummary();
		return StringUtils::unHtmlSpecialChars($summary);
	}

	/**
	 *	getDetails - construct a string of this message summary (html chars unconverted).
	 *
	 *	@param	User
	 *	@return string
	 */
	function getDetails(User $User) {

		$details = $this->news->getDetails();
		return StringUtils::unHtmlSpecialChars($details);
	}	
	
	/**
	 *	getNotificationSubject - construct a string of this message subject.
	 *
	 *	@param	User
	 *	@return string
	 */
	function getNotificationSubject(User $User) {
		
		$subject = '['.GFConf::get('system.name').']['.$this->news->getProject()->getProjectName().'] '. StringUtils::unHtmlSpecialChars($this->getSummary($User));
		return $subject;
	}

	/**
	 *	getUsers - get an array of user objects
	 *	@return array
	 */
	public function getUsers() {
		return $this->news->getMonitoringUsers();
	}

	/**
	 *	getSection - retrieves the section name of the news
	 *	@return string
	 */	
	function getSection() {
		return News::SECTION;
	}
	
	/**
	 *	getRefId - retrieves the id of the news that the notification belongs to
	 *	@return int
	 */	
	function getRefId() {
		return $this->news->getPrimaryKey();
	}
	
}

?>
