<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Daniel Perez	<daniel@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('plugin/GFPlugin.php');
require_once('plugin/ActionManager.php');
require_once('plugin/PluginResult.php');
require_once('plugins/forum/lib/ForumPeer.php');
require_once('core/Language.php');
require_once('NewsPeer.php');
require_once('NewsCountPeer.php');
require_once('lib/GFExportImportUtils.php');

class NewsPlugin implements GFPlugin {

	protected $project;
	
	CONST SECTION = 'news';

	/*function __construct($project) {
		$this->project = $project;
	}*/
	function __construct() {
	}	
	
	/**
	 *	this is called when the first Action is called. Action manager 
	 *	gets the listeners for each plugin.
	 */
	public function initialize() {
		ActionManager::registerListeners($this, $this->getListeners());
	}

	/**
	 *	Different types of plugins are possible -
	 *	GFPlugin::SITE_PLUGIN = 'site'
	 *	GFPlugin::PROJECT_PLUGIN = 'project'
	 *	GFPlugin::USER_PLUGIN = 'user'
	 */
	public function getType() {
		return GFPlugin::PROJECT_PLUGIN;
	}	
	
	/**
	 *	get the name of this plugin. example: 'tracker'
	 *	@return	string
	 */
	public function getName() {
		return 'news';
	}

	/**
	 *	get the array of action names that this plugin answers to.
	 *	@return	array
	 */
	public function getListeners() {
		return array(
			'BUILD_PROJECT_TABS',
			'GET_PROJECT_ADMIN_NAV_ELEMENTS',
			'BUILD_PROJECT_NEWS_SUMMARY',
			'BUILD_SEARCH_COMBO',
			'DO_SEARCH',
			'PROJECT_APPROVE',
			'BUILD_MYPAGE_ELEMENTS',
			'MONITOR_CHECK_PERMISSIONS',
			'SITE_ADMIN_INDEX',
			'PROJECT_FULLEXPORT',
			'PROJECT_FULLIMPORT',
			'PROJECT_TPLEXPORT',
			'PROJECT_TPLIMPORT',
			'USER_PROJECT_DELETE',
			'USERGROUP_USER_REMOVE',
	//		'BUILD_FRONTPAGE_ELEMENTS'
			'BUILD_MYPAGE_MONITORS'
//			'GET_NEWS_NAV_ELEMENTS'
			);
	}

	/**
	 * returns the part of the html text that goes into the my page area, for monitored items
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageMonitors($params) {	
		$con = Propel::getConnection(NewsPeer::DATABASE_NAME);
		$theme = $params['theme'];

		//get the assigned items for the user
		$user = Session::getSession()->getUser();
		$user_id = $user->getPrimaryKey();
/*
		// news forums
		$sql = "select f.forum_id, f.forum_name, p.project_id from forum f, monitor m, news n, project p 
		where m.user_id = '$user_id' and f.section='". $this->getName() . "' and m.section='forum' 
		 and m.ref_id = f.forum_id and n.news_id = f.ref_id and n.project_id = p.project_id";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
		
		$theader = array();
		$theader[] = Language::getSessText('My.NewsForums');
// TODO -> create a page or something to "stop monitoring" and put a link in the table
		$value = $theme->tableTop($theader);		
		
		if ($rs->getRecordCount()>0) {
			while($rs->next()) {
				$trow = array();
				$forum_name = $rs->getString('forum_name');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'ForumThreadBrowse','forum','',array('forum_id'=>$rs->getInt('forum_id'))) . '" >' .  $forum_name . '</a>';
				$value .= $theme->tableRow($trow);
			}
		} else {
			$trow = array();
			$trow[] = Language::getSessText('My.NoMonitoredNewsForums');
			$value .= $theme->tableRow($trow);
		}
		$value .= $theme->tableBottom();
*/
		// news forums
		$sql = "SELECT p.project_name, p.project_id FROM monitor m, project p 
		WHERE m.user_id = '$user_id' AND m.section='news' AND m.ref_id = p.project_id";
		$stmt = $con->createStatement();
		$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);

		$value = '<h3>'.Language::getSessText('News.Monitoring').'</h3>';

		$theader = array();
		$theader[] = Language::getSessText('News.Project');
		$theader[] = '';
		$value .= $theme->tableTop($theader);		
	
		if ($rs->getRecordCount()>0) {
			while($rs->next()) {
				$trow = array();
				$tmod = array();
				$project_name = $rs->getString('project_name');
				$project_id = $rs->getString('project_id');
				$trow[] = '<a href="' . GFUrl::getUrl()->projectURL(ProjectPeer::retrieveByPk($rs->getInt('project_id')),'','news','',array()) . '" >' .  $project_name . '</a>';
				$tmod[] = 'width="30%"';
				$trow[] = '<a href="'.GFUrl::getUrl()->getMonitorUrl(NewsPlugin::SECTION, $project_id , null, false).'" >'.Language::getSessText('WikiPage.StopMonitoring').'</a>';					
				$tmod[] = '';
				$value .= $theme->tableRow($trow, $tmod);
			}
		} else {
			$value .= $theme->tableFullRow(Language::getSessText('My.NoMonitoredNewsForums'));
		}
		$value .= $theme->tableBottom();
		
		$result = new PluginResult($this);
		$result->setResult($value);
		return $result;		
	}		

	/**
	 *	handle action has a case statement to redirect the action to the appropriate function.
	 *	@param string
	 *	@param mixed
	 *	@return PluginResult
	 */
	public function handleAction($listenerName, $params) {
		switch ($listenerName) {
			case 'BUILD_PROJECT_TABS': {
				return $this->_buildMainTabsListener($params);
				break;
			}
			case 'GET_PROJECT_ADMIN_NAV_ELEMENTS': {
				return $this->getProjectAdminNavElements($params);
				break;
			}
			case 'BUILD_PROJECT_NEWS_SUMMARY': {
				return $this->_buildProjectSummary($params);
				break;
			}
			case 'BUILD_SEARCH_COMBO': {
				return $this->buildSearchOptions($params);
				break;
			}
			case 'DO_SEARCH': {
				return $this->getSearchResults($params);
				break;
			}				
			case 'PROJECT_APPROVE': {
				return $this->approveActions($params);
				break;
			}			
			case 'BUILD_MYPAGE_ELEMENTS': {
				return $this->buildMyPageElements($params);
				break;
			}
			case 'BUILD_MYPAGE_MONITORS': {
				return $this->buildMyPageMonitors($params);
				break;
			}
			case 'MONITOR_CHECK_PERMISSIONS': {
				return $this->monitorCheckPermissions($params);
				break;
			}
			case 'SITE_ADMIN_INDEX': {
				return $this->getPluginSiteAdminelements($params);
				break;
			}
			case 'PROJECT_FULLIMPORT':
			case 'PROJECT_FULLEXPORT': {
				return GFExportImportUtils::getSpecData($this);
				break;
			}
			case 'PROJECT_TPLIMPORT':
			case 'PROJECT_TPLEXPORT': {
				return GFExportImportUtils::getSpecData($this, 'specDataTpl.xml');
				break;
			}
			case 'USER_PROJECT_DELETE': {
				return $this->deleteUserMonitors($params);
				break;
			}
			case 'USERGROUP_USER_REMOVE': {
				return $this->deleteUsergroupUserMonitors($params);
				break;
			}
			/*case 'BUILD_FRONTPAGE_ELEMENTS': {
				return $this->buildFrontPageElements($params);
				break;
			}*/
		}
	}


	private function deleteUsergroupUserMonitors($usergroup_user) {
		$con = Propel::getConnection();		

		$project_ids = $usergroup_user->getProjectMonitorsToDelete();
		$user_id = $usergroup_user->getMemberUserId();

		if ($project_ids != '') {
			$stmt = $con->createStatement();
			//Delete monitor for the projects
			$sql = "DELETE FROM monitor WHERE section='".NewsPlugin::SECTION."' AND ref_id IN 
							(
								SELECT project_id FROM project WHERE
								project_id IN (".$project_ids.")
								AND
								project.is_public = false
							)
							AND user_id = ".$user_id;
			$stmt->executeQuery($sql);
		}		
	}


	private function deleteUserMonitors($user_project) {
		$user_id = $user_project->getUserId();
		$project = $user_project->getProject();
		if (is_object($project)) {
			if (!$project->isPublic()) {
				$con = Propel::getConnection();
				$stmt = $con->createStatement();
		
				$sql = "DELETE FROM monitor WHERE section='".NewsPlugin::SECTION."' AND ref_id = ".$project->getProjectId()."
								AND user_id = ".$user_id;
		
				$stmt->executeQuery($sql);
			}
		}
	}

	/**
	 * returns the part of the html text that goes into the my page area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildMyPageElements($params) {	

	}
	
	/**
	 * returns the part of the html text that goes into the front page
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 * /
	private function buildFrontPageElements($params) {	
		$c = new Criteria();
		$c->add(NewsPeer::IS_APPROVED,true);
		$approved = NewsPeer::doSelect($c);
		$result = new PluginResult($this);
		$template = $params['template'];
		$res = $template->infoTableMiddle(Language::getSessText('News.LatestNews'));
		$res .= '<ul>';
		foreach ($approved as $approved) {
			$res .= '<li> (' . $approved->getPostDate() . ') - ' . $approved->getSummary() . '</li>';
		}
		$result->setResult($res);
		return $result;
	}	

	/**
	 * Build Admin Site link
	 *
	 */	
	private function getPluginSiteAdminElements($params) {
		$arr = array(
			'title' => Language::getSessText('News.TabString'),
			'elements' => array('line' => '<a href="'.GFUrl::getUrl()->baseURL('NewsApprove','','admin').'">'.Language::getSessText('News.ApproveRejectFPNews').'</a>')
		);
		$pr = new PluginResult($this);
		$pr->setResult($arr);
		return $pr;
	}
		
	
	/**
	 * returns the part of the html text that goes into the site admin area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */
	private function buildSiteAdminElements($params) {	
		$result = new PluginResult($this);
		$ret = '<p><strong>' . Language::getSessText('News.TabString') .'</strong></p>
				<ul>
					<li><a href="'. GFUrl::getUrl()->projectURL($params['project'],'',$this->getName(),'admin') .'">'. Language::getSessText('News.Approve') . '</a></li>
				</ul>';
		$result->setResult($ret);
		return $result;		
	}	

	/**
	 * returns the part of the html text that goes into the project public area
	 *
	 * @param array $params needed params to build the html
	 * @return string	the builded html result
	 */	
	private function _buildProjectSummary($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);

		$project = $params['project'];
		$theme = $params['theme'];
		
		$theader=array(Language::getSessText('Project.News'));
		$value=$theme->tableTop($theader);

		$c = new Criteria();
		$c->setLimit(5);
		$c->add(NewsPeer::PROJECT_ID,$project->getPrimaryKey());
		$c->addDescendingOrderByColumn(NewsPeer::POST_DATE);
		$news = NewsPeer::doSelect($c);
		
		$url = GFUrl::getUrl();

		if (count($news) < 1) {
			$value .= $theme->tableFullRow(Language::getSessText('News.NoneFound'));
		}
		foreach ($news as $new) {
			$trow=array();
			$trow[]='<strong><a href="'.$url->projectUrl($project, 'NewsThreadView', 'news', '', array('id' => $new->getPrimaryKey())).'">'.$new->getSummary().'</a></strong><br />'.
			$new->getUser()->getUserLink().'<br />'.
			$new->getPostDate('Y-m-d') .'<p>'.
			nl2br(StringUtils::addHtmlLinks($new->getShortDetails())).'</p>';
//$new->getUser()->getUserLink() .' - '. $new->getPostDate();
			$value .= $theme->tableRow($trow,array(),true);
			unset($trow);
		}

		$value .=$theme->tableBottom();
		//$value .= $theme->tableBottom();
		
		$result->setResult($value);
		return $result;
	}	
	
	/**
	 *	take a search object and handle the search internally.
	 *	@param	GFSearch
	 *	@return PluginResult
	 */
	public function search(GFSearch $search) {
		if (!$search->getProjectId()) {
			return $this->globalSearch($search);
		}

		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND news.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			$sql = "SELECT headline(news.summary, q) as summary,
			news.post_date,
			news.submitted_by,
			news.news_id
			FROM news, news_idx, to_tsquery('$words') AS q 
			WHERE (news.project_id='".$search->getProjectId()."'
			$date_sql
			AND news_idx.news_id = news.news_id
			AND (news_idx.vectors @@ q))
			ORDER BY rank(news_idx.vectors, q) DESC";
			
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_ASSOC);
			$template = $search->getTemplate();
			if ($rs->getRecordCount()>0) {
				$ret = '';
				$project = ProjectPeer::retrieveByPk($search->getProjectId());
				$ret = '<p><h3>'.Language::getSessText('News.SearchResultsFor').'</h3></p>';
				$ret .= $template->tableTop(array(Language::getSessText('News.SummaryName'),Language::getSessText('News.SubmittedByName'),Language::getSessText('News.PostDateName')));
				$i = 0;
				while ($rs->next()) {
					$newsid = $rs->getInt('news_id');
					$trow = array();
					$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project,'NewsThreadView',$this->getName(),'',array('id'=>$newsid)) . '">' . $rs->getString('summary').'</a>';
					$trow[] = UserPeer::retrieveByPK($rs->getInt('submitted_by'))->getName();
					$trow[] = $rs->getDate('post_date');
					$ret .= $template->tableRow($trow);
				}
				$ret .= $template->tableBottom();
				return $ret;
			} else {
				// cero results for search
				return '<p><h3>'.Language::getSessText('News.SearchResultsFor').'</h3></p>' . $template->tableTop(array(Language::getSessText('News.SummaryName'),Language::getSessText('News.SubmittedByName'),Language::getSessText('News.PostDateName'))) . $template->tableFullRow( Language::getSessText('Search.NoResults') ) . $template->tableBottom();
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
	}

	/**
	 *	Searches across all projects
	 *	@param	GFSearch
	 *	@return string
	 */
	private function globalSearch($search) {
		if (GFConf::get('system.dbtype') == 'postgres') {		
			$con = Propel::getConnection(UserPeer::DATABASE_NAME);
			$words = $search->getWordsForQuery();

			if ($search->getStartDate()) {
				$date_sql = " AND news.post_date BETWEEN '".$search->getStartDate()."' AND '".$search->getEndDate()."' ";
			} else {
				$date_sql='';
			}

			$sql = "SELECT project.project_name, 					-- 1
				project.unix_name, 									-- 2
				headline(news.summary, q) as headline_summary,		-- 3
				news.*												-- 4
			FROM news, news_idx, to_tsquery('$words') AS q, project
			WHERE (news.project_id=project.project_id
			$date_sql
			AND news_idx.news_id = news.news_id
			AND (news_idx.vectors @@ q))
			ORDER BY rank(news_idx.vectors, q) DESC";
			
			$stmt = $con->createStatement();
			$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
			$template = $search->getTemplate();
			
			$ret = '';
			$project = ProjectPeer::retrieveByPk($search->getProjectId());
			$ret = '<p><h3>'.Language::getSessText('News.SearchResultsFor').'</h3></p>';
			$ret .= $template->tableTop(array(
							Language::getSessText('Project.ListNameTitle'),
							Language::getSessText('News.SummaryName'),
							Language::getSessText('News.SubmittedByName'),
							Language::getSessText('News.PostDateName'))
						);
			$count = 0;
			while ($rs->next()) {
				$news = new News();
				$news->hydrate($rs, 4);
				
				if (!$news->userCanRead()) {
					continue;
				}
				
				$count++;
				$project_unix_name = $rs->getString(2);
				$newsid = $news->getNewsId();
				$trow = array();
				$trow[] = '<a href="'.GFUrl::getUrl()->projectUrl($project_unix_name).'">'.$rs->getString(1).'</a>';
				$trow[] = '<a href="'.GFUrl::getUrl()->projectURL($project_unix_name,'NewsThreadView',$this->getName(),'',array('id'=>$newsid)) . '">' . $rs->getString(3).'</a>';
				$trow[] = UserPeer::retrieveByPK($news->getSubmittedBy())->getName();
				$trow[] = $news->getPostDate();
				$ret .= $template->tableRow($trow);
			}
			$ret .= $template->tableBottom();

			if ($count > 0) {
				return $ret;
			} else {
				// cero results for search
				return '<p><h3>'.Language::getSessText('News.SearchResultsFor').'</h3></p>' . $template->tableTop(array(Language::getSessText('News.SummaryName'),Language::getSessText('News.SubmittedByName'),Language::getSessText('News.PostDateName'))) . $template->tableFullRow( Language::getSessText('Search.NoResults') ) . $template->tableBottom();
			}
		} elseif (GFConf::get('system.dbtype') == 'mysql') {
			
		} elseif (GFConf::get('system.dbtype') == 'oracle') {
			
		}
	}

	
	/**
	 * returns the options to print in the combobox
	 *
	 * @param unknown_type $params
	 * @return unknown
	 */
	private function buildSearchOptions($params) {
		$result = new PluginResult($this);
		
		if (isset($params['project'])) {
			if (!$params['project']->usesPlugin($this->getName())) {
				return;
			}
			$text = Language::getSessText('News.ThisProjectNews');
		} else {
			$text = Language::getSessText('News.AllProjectsNews');
		}
		
		$value = array();
		// only give result if were in a project
		$value[] = array(
			'value'	=> $this->getName(),
			'display'	=> $text
			);
		
		$result->setResult($value);
		return $result;		
	}
	
	/**
	 * returns the search results for the given input
	 *
	 * @param array $params
	 * @return resultset
	 */
	private function getSearchResults($params) {
		if (array_search($this->getName(), $params['search']) !== false) {
			$result = new PluginResult($this);
			$searcher = new GFSearch($params);
			$result->setResult($this->search($searcher));
			return $result;
		}
	}		
	
	/**
	 * returns an array of the fields where we want the search to be done upon
	 *
	 */
	private function getSearchFields() {
		return array(NewsPeer::SUMMARY,NewsPeer::DETAILS);
	}	
	
	private function _buildMainTabsListener($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$project = $params['project'];
		
		$result = new PluginResult($this);
		$value = array(
				'text'	=> Language::getSessText('News.TabString'),
				'action'	=> GFUrl::getUrl()->projectURL($project,'','news'),
				'id'	=> 'news'
			);
		$result->setResult($value);
		return $result;
	}	

	/**
	 *      getNewsNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 * / 
	private function getNewsNavElements() {
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'News Title',
			'elements'=> array(
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 1'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 2'),
			array('link'=>'/path/to/file/?action',
				'description'=>'Title 3')
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *      getProjectAdminNavElements - get the local nav elements for the tracker
	 *      @return PluginResult
	 */
	private function getProjectAdminNavElements($params) {
		if (!isset($params['project']) || !is_object($params['project']) || !$params['project']->usesPlugin($this->getName())) {
			return;
		}
		$result = new PluginResult($this);
		$arr = array(
			'title'=>'News',
			'elements'=> array(
			array('link'=>'/gf/project/'.$params['project']->getUnixName().'/news/admin/',
				'description'=>Language::getSessText('News.Admin'))
			)
		);
		$result->setResult($arr);
		return $result;
	}

	/**
	 *	get project uri to be used as root for constructing uris
	 *	@return string
	 */
	function getProjectPluginUri() {
		return GFUrl::getUrl()->projectURL($this->project,'','news');
	}
	
	function enableInProject(Project $project) {
		return;
	}

	function disableFromProject() {
		// unregister the actions
		ActionManager::unregisterListeners($this);
	}
	
	/**
	 * actions to perform on project approval
	 *
	 * @param $params array
	 * @return PluginResult
	 */
	private function approveActions($params) {
		$template_project = $params['template_project'];
		$project = $params['project'];
		
		if (!($project instanceof Project) || !($template_project instanceof Project)) {
			return;
		}
		
	}
	
	/**
	 * Check if the user can monitor certain item
	 */	
	private function monitorCheckPermissions($params) {
		$lang = Session::getSession()->getLanguage();

		$section = $params['section'];
		$ref_id = $params['ref_id'];
		
		if ($section != 'news') return null;		// only deal with news monitor
		$pr = new PluginResult($this);
		
		// DO NOT CONFUSE: ref_id points to the project ID, not to the news
		// ID. This is done because we want to monitor *all* the project's news.
		$project = ProjectPeer::retrieveByPk($ref_id);
		if (is_null($project) || !$project->userCanRead()) {
			$pr->setResult(false);
		} else {
			$pr->setResult(true);
		}

		return $pr;
	}
	
}
