<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Marcelo Mottalli <marcelo@gforgegroup.com>
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('html/TroveFilterRenderer.php');

/**
 * This class manages the Trove forms
 */
class SnippetTroveFilterRenderer extends TroveFilterRenderer {
	private $snippet_section;
	private $snippet_ref_id;
	

	public function __construct($snippet_section, $snippet_ref_id) {
		parent::__construct('snippet');
		$this->snippet_section = $snippet_section;
		$this->snippet_ref_id = $snippet_ref_id;
	}

	protected function prepareCountJoinCriteria(Criteria $c) {
		$c->add(TroveLinkPeer::REF_ID, SnippetPeer::SNIPPET_ID);
		$c->add(SnippetPeer::SECTION, $this->snippet_section);
		$c->add(SnippetPeer::REF_ID, $this->snippet_ref_id);
		$c->addSelectColumn(SnippetPeer::COUNT);
	}
	
	protected function getCategoryResults(Criteria $c) {
		$lang = Language::getSessLang();
		$result = '';
		$theme = Session::getSession()->getTheme();

		$result .= '<br />';

		$isAdmin = Session::getSession()->isSiteAdmin();
		$theader = array();
		$theader[] = $lang->getText('Snippet.NameName');
		$theader[] = $lang->getText('Snippet.CreatedByName');
		if ($isAdmin) {
			$theader[] = $lang->getText('Edit');
			$theader[] = $lang->getText('Delete');
		}
		$result .= $theme->tableTop($theader);


		$c->addJoin(TroveLinkPeer::REF_ID, SnippetPeer::SNIPPET_ID);
		$c->add(SnippetPeer::SECTION, $this->snippet_section);
		$c->add(SnippetPeer::REF_ID, $this->snippet_ref_id);
		$rs = SnippetPeer::doSelectRS($c);		// May get lots of results, so use a recordset
		$count = 0;
		while ($rs->next()) {
			$snippet = new Snippet();
			$snippet->hydrate($rs);
			
			$trove = TroveLinkPeer::getSelectedCategoriesTree('snippet', $snippet->getSnippetId());
			if ($this->matchesFilters($trove)) {
				$trow = array();
				$trow[]='<a href="'. GFUrl::getUrl()->currentBaseURL('SnippetBrowse',array('id'=>$snippet->getSnippetID())) . '">'.$snippet->getSnippetName().'</a>';
				$trow[]=$snippet->getUser()->getUserLink();
				if ($isAdmin) {
					$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('SnippetEdit').'&amp;snippet_id='.$snippet->getSnippetID() .'">'. $lang->getText('Edit') .'</a>';
					$trow[]='<a href="'. GFUrl::getUrl()->currentBaseUrl('SnippetDelete').'&amp;snippet_id='.$snippet->getSnippetID() .'&amp;_trove_category_id='. $_GET['_trove_category_id'] .'">'. $lang->getText('Delete') .'</a>';
				}
		
				$result .= $theme->tableRow($trow);
				$count++;
			}
		}
		
		if ($count == 0) {
			$result .= $theme->tableFullRow($lang->getText('Snippet.NoneFound'));
		}
		
		$result .= $theme->tableBottom(array(GFUrl::getUrl()->currentBaseUrl('SnippetAdd') => $lang->getText('Snippet.AddNew')));
		
		return $result;
	}
	
	protected function getCountString($category_count, $children_count) {
		return Language::getSessText('Snippet.TroveCount', array($category_count, $children_count));
	}
}
?>
