<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
//
// This code creates a new Tracker and saves it to the 
// database
//
require_once('plugins/tracker/lib/Tracker.php');
require_once('Validator.php');
require_once('exceptions/InvalidDataException.php');
require_once('exceptions/PermissionDeniedException.php');
require_once('core/Language.php');

if ($_SERVER['REQUEST_METHOD'] != 'POST') {
	throw new PermissionDeniedException();
}

$session = Session::getSession();
$Lang = $session->getLanguage();

$Project = GFUrl::getUrl()->getProject();
$Tracker = new Tracker();

$errors = array();
$language = Language::getSessLang();

if (isset($_POST['clone_tracker']) && !$_POST['clone_tracker_id']) {
	//User chose to clone tracker but didn't select any tracker
	$errors[] = $language->getText('Tracker.SelectCloneTracker');
}

if (count($errors) == 0) {
	$Tracker->setProject( $Project );
	$Tracker->setTrackerName( Validator::sanitizeString($_POST['tracker_name']) );
	$Tracker->setDescription( Validator::sanitizeString($_POST['description']) );
	$Tracker->setIsPublic( isset($_POST['is_public']) ? true : false );
	$Tracker->setRestrictBrowse( isset($_POST['restrict_browse']) ? true : false );
	$Tracker->setEmailAllUpdates( isset($_POST['email_all_updates']) ? true : false );
	$Tracker->setEmailAddress( Validator::sanitizeString($_POST['email_address']) );
	$duedays=isset($_POST['due_period']) ? Validator::sanitizeInt($_POST['due_period']) : 15;
	$Tracker->setDuePeriod( $duedays*60*60*24 );
	$Tracker->setSubmitInstructions( Validator::sanitizeString($_POST['submit_instructions']) );
	$Tracker->setBrowseInstructions( Validator::sanitizeString($_POST['browse_instructions']) );
	
	$Tracker->setShowDependenciesTab(isset($_POST['show_dependencies_tab']) ? true : false);
	$Tracker->setShowDuplicatesTab(isset($_POST['show_duplicates_tab']) ? true : false);
	$Tracker->setShowAssociationsTab(isset($_POST['show_associations_tab']) ? true : false);
	$Tracker->setShowTimeTrackingTab(isset($_POST['show_time_tracking_tab']) ? true : false);
	$Tracker->setShowTagsTab(isset($_POST['show_tags_tab']) ? true : false);

	$Tracker->setDatatype( isset($_POST['datatype']) ? Validator::sanitizeInt($_POST['datatype']) : Tracker::DATATYPE_TRACKER );
	
	if (isset($_POST['clone_tracker'])) {
		$clone_tracker = TrackerPeer::retrieveByPk($_POST['clone_tracker_id']);
	} else {
		$clone_tracker = null;
	}
	
	try {
		$Tracker->save();
		if (!is_null($clone_tracker)) {
			// copy the fields from some other tracker
			$Tracker->cloneFields($clone_tracker);
		} else {
			// if we're not cloning the tracker, create some default fields
			$Tracker->buildDefaultFields();
		}

		GFUrl::getUrl()->redirect(GFUrl::getUrl()->currentBaseURL('TrackerBrowse'), array('feedback' => $language->getText('Tracker.AddFeedback')));
	} catch (InvalidDataException $ex) {
		$errors = array_merge($errors, $ex->getErrors());		// attach object errors to validation errors
		$feedback = '* '.implode('<br />* ', $errors);
		include('TrackerAdd.php');
	}
} else {
	$feedback = '* '.implode('<br />* ', $errors);
	include('TrackerAdd.php');
}
	
?>
