<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id: Layout.php 270 2005-12-31 03:02:47Z tperdue $
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */
require_once('html/MyTemplate.php');
require_once('lib/core/FilesystemPeer.php');
require_once('lib/core/MyReport.php');
require_once('lib/core/MyReportPeer.php');
require_once('lib/core/MyReportFieldFilter.php');
require_once('lib/core/MyReportFieldDisplay.php');
require_once('lib/core/MyReportFieldFilterPeer.php');
require_once('lib/core/MyReportFieldDisplayPeer.php');
require_once('lib/core/MyReportTracker.php');
require_once('lib/core/MyReportTrackerPeer.php');
require_once('lib/core/MyReportProject.php');
require_once('lib/core/MyReportProjectPeer.php');
require_once('lib/core/MyReportFieldValue.php');
require_once('lib/core/MyReportFieldValuePeer.php');


$session=Session::getSession();
if (!$session->isLoggedIn()) {
	throw new NotLoggedInException();
}
$user = Session::getSession()->getUser();

if(isset($_REQUEST['feedback'])) $feedback = $_REQUEST['feedback'];
if(isset($_REQUEST['report_id'])) {
	$report_id = Validator::sanitizeInt($_REQUEST['report_id']);
	// Fetch report data
	if($report_id != 0) {
		$report = MyReportPeer::retrieveByPK($report_id);
		if(!is_object($report)) {
			throw new InvalidDataException();
		}
		// Check if the user can view the report
		if (!$report->userCanRead()) {
			throw new PermissionDeniedException();
		}
	}
	else {
		throw new InvalidDataException();
	}
}
else {
	throw new InvalidDataException();
}
$Lang = $session->getLanguage();

$separator = ',';

$filename = StringUtils::removeInvalidChars(str_replace(' ', '_',$report->getReportName()));

$tracker_item_ids = $report->getTrackerItemIds();

$theader = array();
$headers = array();

$theader[] = Language::getSessText('Report.TrackerItemId');

if(is_array($tracker_item_ids) && count($tracker_item_ids)>0) {
	//$tracker_items = TrackerItemPeer::retrieveByPKs($tracker_item_ids);
	$display_data = $report->getDisplayData($tracker_item_ids);
}
else {
	$display_data = array();
}
	
$crit = new Criteria();
$crit->add(MyReportFieldDisplayPeer::MY_REPORT_ID, $report_id);
$crit->addAscendingOrderByColumn(MyReportFieldDisplayPeer::COLUMN_ORDER);
$display_field_names = MyReportFieldDisplayPeer::doSelect($crit);

foreach($display_field_names as $display_field_name) {
	switch($display_field_name->getType()) {
		case MyReport::FIELDTYPE_ASSIGNEES:
			$theader[$display_field_name->getFieldName()] = trim(htmlspecialchars(Language::getSessText('TrackerItem.AssignedTo')));
			break;
		case MyReport::FIELDTYPE_SUMMARY:
			$theader[$display_field_name->getFieldName()] = trim(htmlspecialchars(Language::getSessText('TrackerItem.SummaryName')));
			break;
		case MyReport::FIELDTYPE_DETAILS:
			$theader[$display_field_name->getFieldName()] = trim(htmlspecialchars(Language::getSessText('TrackerItem.DetailsName')));
			break;
		case MyReport::FIELDTYPE_PRIORITY:
			$theader[$display_field_name->getFieldName()] = trim(htmlspecialchars(Language::getSessText('TrackerItem.PriorityName')));
			break;
		case MyReport::FIELDTYPE_OPENDATE:
			$theader[$display_field_name->getFieldName()] = Language::getSessText('TrackerItem.OpenDateName');
			break;
		case MyReport::FIELDTYPE_CLOSEDATE:
			$theader[$display_field_name->getFieldName()] = Language::getSessText('TrackerItem.CloseDateName');
			break;
		case MyReport::FIELDTYPE_LASTMODIFIEDDATE:
			$theader[$display_field_name->getFieldName()] = Language::getSessText('TrackerItem.LastModifiedDateName');
			break;
		case MyReport::FIELDTYPE_TRACKERNAME:
			$theader[$display_field_name->getFieldName()] = Language::getSessText('Tracker.TabString');
			break;
		case MyReport::FIELDTYPE_PROJECTNAME:
			$theader[$display_field_name->getFieldName()] = Language::getSessText('Common.Project');
			break;
		default:
			$theader[$display_field_name->getFieldName()] = trim(htmlspecialchars($display_field_name->getFieldName()));
			break;
	}
	$headers[] = $display_field_name->getFieldName();
}

$dom = new DOMDocument('1.0', 'iso-8859-1');
$domReport = $dom->createElement('Report');
$domReportNode = $dom->appendChild($domReport);
foreach($display_data as $tracker_item_id => $display_fields) {
	$domTrackerItem = $dom->createElement('TrackerItem');
	$domTrackerItem->setAttribute('id', $tracker_item_id);
	$domTrackerItemNode = $domReportNode->appendChild($domTrackerItem);

	$tracker_item = TrackerItemPeer::retrieveByPk($tracker_item_id);
	$trow[] = $tracker_item_id;

	foreach($headers as $header) {
		if(isset($display_fields[$header]) && count($display_fields[$header])>0) {
			$trow_str = '';
			foreach($display_fields[$header] as $row_str ) {
				$trow_str .= str_replace('|','',$row_str) . '|';
			}
			$trow_str = substr($trow_str,0, strlen($trow_str)-1);
		}
		else {
			$trow_str = '';
		}		
		$domCDATA = $dom->createCDATASection($trow_str);
		$domItem = $dom->createElement('field');
		if(isset($theader[$header])) {
			$domItem->setAttribute('name', $theader[$header]);
		}
		$domItem->appendChild($domCDATA);
		$domTrackerItemNode->appendChild($domItem);
	}
	
}
header("Content-Type: text/xml");
echo $dom->saveXML();
?>
