<?php
/**
 * Copyright (c) 2005-2008 GForge, LLC
 *
 * @version   $Id$
 * @author Tim Perdue tim@gforge.org
 * @date 2006-02-27
 *
 * This file is part of GForge.
 *
 * See LICENSE for details. This file is part of GForge AS and may not be
 * redistributed without written permission of GForge, LLC
 */

require_once('core/ProjectPeer.php');
require_once('session/Session.php');
require_once('core/AuditTrail.php');
require_once('plugin/GFPlugin.php');
require_once('html/HTMLWidget.php');


$session = Session::getSession();
$project = GFUrl::getUrl()->getProject();
$project_id = $project->getProjectId();
$Language = $session->getLanguage();

if (!isset($feedback)) {
	$feedback = '';
}

require_once('html/templates/ProjectAdminTemplate.php');
$template = new ProjectAdminTemplate();
$params = array(
		'project' => $project,
		'feedback' => $feedback,
		'title' => Language::getSessText('ProjectReporting.AuditTrail')
	);
$template->header($params);
$begin=((isset($_REQUEST['start_date']) && strlen($_REQUEST['start_date']) > 5) ? $_REQUEST['start_date'] : 0 );
$end=((isset($_REQUEST['end_date']) && strlen($_REQUEST['end_date']) > 5) ? $_REQUEST['end_date'] : 0 );

if ($begin=="0") {
	$begin = (time()-(30*86400));
} else {
	$begin = strtotime($begin);
}
if ($end=="0") {
	$end = time();
} else {
	$end=strtotime($end)+86399;
}
if ($begin > $end) {
	$endtmp=$end;
	$end=$begin;
	$begin=$endtmp;
}

$con = Propel::getConnection(AuditTrailPeer::DATABASE_NAME);
$stmt = $con->createStatement();

// We put everything that's possible in one big query so we don't have to make many subqueries later
$subquery = "			-- Changes in project fields
			SELECT audit_trail.*, \"user\".unix_name, \"user\".firstname, \"user\".lastname, 
				'' AS extra1, '' AS extra2
			FROM audit_trail
			NATURAL JOIN \"user\"
			WHERE audit_trail.section='project'
			AND audit_trail.ref_id=".$project->getProjectId()."
			AND audit_trail.change_date BETWEEN '".date('Y-m-d',$begin)." 00:00:00' AND '".date('Y-m-d',$end)." 23:59:59'
		UNION
			-- Changes in users
			SELECT audit_trail.*, \"user\".unix_name, \"user\".firstname, \"user\".lastname, 
				new_user.unix_name AS extra1, old_user.unix_name AS extra2
			FROM audit_trail
			NATURAL JOIN \"user\"
			LEFT OUTER JOIN \"user\" AS new_user ON (audit_trail.new_value)::int = new_user.user_id
			LEFT OUTER JOIN \"user\" AS old_user ON (audit_trail.old_value)::int = old_user.user_id
			WHERE audit_trail.section='user_project' AND audit_trail.ref_id=".$project->getProjectId()."
			AND audit_trail.change_date BETWEEN '".date('Y-m-d',$begin)." 00:00:00' AND '".date('Y-m-d',$end)." 23:59:59'
		UNION
			SELECT audit_trail.*, \"user\".unix_name, \"user\".firstname, \"user\".lastname, 
				'' AS extra1, '' AS extra2
			FROM audit_trail
			NATURAL JOIN \"user\"
			WHERE audit_trail.section='user_project_role'
			AND audit_trail.ref_id=".$project->getProjectId()."
			AND audit_trail.change_date BETWEEN '".date('Y-m-d',$begin)." 00:00:00' AND '".date('Y-m-d',$end)." 23:59:59'
		";


$sql = "SELECT COUNT(*) FROM ($subquery) AS q1";

$rs = $stmt->executeQuery($sql, ResultSet::FETCHMODE_NUM);
$rs->next();
$total = $rs->getInt(1);
$paginator = new Paginator('audit', -1, $total, 'change_date', array('change_date'=>''));

$sql = "SELECT * FROM ($subquery) AS q1 " . $paginator->getSQL();

$rs = $stmt->executeQuery($sql);
?>

<script language="javascript" type="text/javascript" src="/datetimepicker.js">
</script>
<br />
<table border="0" cellspacing="0" cellpadding="3" bgcolor="#E0E0E0">
<form action="<?php echo GFUrl::getUrl()->currentURL() ?>" method="post">
<tr>
	<td><strong><?php echo $Language->getText('ProjectActivity.StartDate') ?></strong></td>
	<td><strong><?php echo $Language->getText('ProjectActivity.EndDate') ?></strong></td>
	<td>&nbsp;</td>
</tr>
<tr>
	<td valign="top"><input id="startdate" name="start_date" value="<?php echo date('M d Y',$begin); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('startdate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
	<td valign="top"><input id="enddate" name="end_date" value="<?php echo date('M d Y',$end); ?>" size="11" maxlength="11" /><a href="javascript:NewCal('enddate','MMMddyyyy')"><img src="/themes/gforge5/images/cal.gif" width="16" height="16" border="0" alt="Pick a date"/></a></td>
	<td valign="top"><?php echo HTMLWidget::button("submit", $Language->getText('Common.Submit'), "submit"); ?></td>
</tr>
</form>
</table>
<br />
<?php

$titles = array(
		$Language->getText('AuditTrail.ChangeDateName'),
		$Language->getText('AuditTrail.Description'),
		$paginator->insertLimitOptions($Language->getText('AuditTrail.By'))
	);
echo $template->tableTop($titles);
$c = 0;
while ($rs->next()) {
	$c++;
	$date = $rs->getDate('change_date');
	$time = $rs->getTime('change_date');
	$section = $rs->getString('section');
	$field_name = $rs->getString('field_name');
	$old_value = $rs->getString('old_value');
	$new_value = $rs->getString('new_value');
	$extra1 = $rs->getString('extra1');
	$extra2 = $rs->getString('extra2');
	
	$unix_name = $rs->getString('unix_name');
	$firstname = $rs->getString('firstname');
	$lastname = $rs->getString('lastname');
	
	$description = '';
	
	switch ($section) {
	case 'project':
		if ($field_name == 'homepage_url') {
			$old_value = (empty($old_value) ? '('.$Language->getText('Common.None').')' : '<a href="'.$old_value.'">'.$old_value.'</a>');
			$new_value = (empty($new_value) ? '('.$Language->getText('Common.None').')' : '<a href="'.$new_value.'">'.$new_value.'</a>');
			$description = $Language->getText('AuditTrail.Project.HomepageUrl', array($old_value, $new_value));
		} elseif ($field_name == 'project_name') {
			$description = $Language->getText('AuditTrail.Project.ProjectName', array($old_value, $new_value));
		} elseif ($field_name == 'description') {
			$old_value = (empty($old_value) ? '('.$Language->getText('Common.None').')' : StringUtils::trimString($old_value, 50));
			$new_value = (empty($new_value) ? '('.$Language->getText('Common.None').')' : StringUtils::trimString($new_value, 50));
			$description = $Language->getText('AuditTrail.Project.ProjectName', array($old_value, $new_value));
		} elseif ($field_name == 'is_public') {
			$old_value = ($old_value ? $Language->getText('Project.IsPublicName') : $Language->getText('Project.IsPrivateName'));
			$new_value = ($new_value ? $Language->getText('Project.IsPublicName') : $Language->getText('Project.IsPrivateName'));
			$description = $Language->getText('AuditTrail.Project.IsPublic', array($old_value, $new_value));
		} elseif ($field_name == 'status') {
			$old_value = Project::getStatusName($old_value);
			$new_value = Project::getStatusName($new_value);
			$description = $Language->getText('AuditTrail.Project.Status', array($old_value, $new_value));
		} elseif ($field_name == 'rolesetting') {
			$description = $Language->getText('AuditTrail.RoleSetting', array($new_value));
		} elseif ($field_name == 'observersetting') {
			$description = $Language->getText('AuditTrail.ObserverSetting');
		} else {
			$params = array('project' => $project, 'audit_trail' => $rs);
			$prs = ActionManager::notify('PROJECT_AUDIT_TRAIL', $params);
			$description = '';
			foreach ($prs as $result) {
				$description .= $result->getResult();
			}
			
		}
		break;
	case 'user_project':
		if (!empty($extra1) && empty($extra2)) {
			$user_url = '<a href="'.GFUrl::getUrl()->userUrl($extra1).'">'.$extra1.'</a>';
			$description = $Language->getText('AuditTrail.UserProject.Added', array($user_url));
		} elseif (empty($extra1) && !empty($extra2)) {
			$user_url = '<a href="'.GFUrl::getUrl()->userUrl($extra2).'">'.$extra2.'</a>';
			$description = $Language->getText('AuditTrail.UserProject.Removed', array($user_url));
		} else {
			continue;
		}
		break;
	case 'user_project_role':
		if (!empty($new_value) && empty($old_value)) {
			if (!preg_match('/([0-9]+),([0-9]+)/', $new_value, $matches)) {
				continue;
			}
			
			list($foo, $user_id, $role_id) = $matches;
			$user = UserPeer::retrieveByPk($user_id);
			$role = RolePeer::retrieveByPk($role_id);
			if (!$user || !$role) {
				continue;
			}
			
			$user_url = '<a href="'.GFUrl::getUrl()->userUrl($user->getUnixName()).'">'.$user->getUnixName().'</a>';
			$description = $Language->getText('AuditTrail.UserProjectRole.Added', array($role->getRoleName(), $user_url));
		} elseif (empty($new_value) && !empty($old_value)) {
			if (!preg_match('/([0-9]+),([0-9]+)/', $old_value, $matches)) {
				continue;
			}
			
			list($foo, $user_id, $role_id) = $matches;
			$user = UserPeer::retrieveByPk($user_id);
			$role = RolePeer::retrieveByPk($role_id);
			if (!$user || !$role) {
				continue;
			}
			
			$user_url = '<a href="'.GFUrl::getUrl()->userUrl($user->getUnixName()).'">'.$user->getUnixName().'</a>';
			$description = $Language->getText('AuditTrail.UserProjectRole.Removed', array($role->getRoleName(), $user_url));
		} else {
			continue;
		}
		break;
	default:
		continue;
	}
	
	$by = "<a href='".GFUrl::getUrl()->userUrl($unix_name)."'>$firstname $lastname</a>";
	
	$row = array(
		$date.' '.$time,
		$description,
		$by
	);
	echo $template->tableRow($row,array(),false);
	
}

if (!$c) {
	echo $template->tableFullRow($Language->getText('AuditTrail.NoneFound'));
}
else {
	//echo $template->listPaginatorRow($paginator);
}

echo $template->tableBottom();

if(isset($paginator) && is_object($paginator)) {
	echo '<div class="paginator">' . $paginator->getPaginator() . '</div><br />';
}


$template->footer();
?>
